#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import unittest
from time import sleep

from libnmap.objects.report import NmapReport
from libnmap.parser import NmapParser
from libnmap.process import NmapProcess


class TestNmapProcess(unittest.TestCase):
    def setUp(self):
        if int(sys.version[0]) == 3:
            self._assertRaisesRegex = self.assertRaisesRegex
        else:
            self._assertRaisesRegex = self.assertRaisesRegexp
        self.fdir = os.path.dirname(os.path.realpath(__file__))

    def test_check_valid_targets(self):
        valid_target_tests = [
            {"value": "127.0.0.1, 1.1.1.1,     2.20.202", "size": 3},
            {"value": ["127.0.0.1", "1.1.1.1", "2.20.202.2"], "size": 3},
            {"value": ["     127.0.0.1", "  1.1.1.1"], "size": 2},
            {"value": "     127.0.0.1,      1.1.1.1  , a", "size": 3},
            {"value": ["192.168.10.0/24", "192.168.0-255.1-254"], "size": 2},
            {"value": ["fe80::a8bb:ccff:fedd:eeff%eth0"], "size": 1},
            {"value": ["my-domain.com", "my-num3r1c-domain.com"], "size": 2},
        ]
        for vtarget in valid_target_tests:
            nmapobj = NmapProcess(targets=vtarget["value"], options="-sP")
            self.assertEqual(vtarget["size"], len(nmapobj.targets))

    def test_check_invalid_targets(self):
        invalid_target_type_tests = [{"a": "bba"}, 5]
        invalid_target_character_tests = ["1.1.1.1$", "invalid_domain.com"]
        invalid_target_dash_tests = ["-invalid-target", "--option"]

        for vtarget in invalid_target_type_tests:
            self._assertRaisesRegex(
                Exception,
                "Supplied target list should be either a string or a list",
                NmapProcess,
                targets=vtarget,
                options="-sP",
            )

        for vtarget in invalid_target_character_tests:
            self._assertRaisesRegex(
                Exception,
                "contains invalid characters",
                NmapProcess,
                targets=vtarget,
                options="-sP",
            )

        for vtarget in invalid_target_dash_tests:
            self._assertRaisesRegex(
                Exception,
                "cannot begin or end with a dash",
                NmapProcess,
                targets=vtarget,
                options="-sP",
            )

    def test_nmap_options(self):
        invalid_options = ["--iflist"]

        for invalid_option in invalid_options:
            self._assertRaisesRegex(
                Exception,
                "unsafe options activated while safe_mode is set True",
                NmapProcess,
                targets="127.0.0.1",
                options=invalid_option,
            )

    def test_missing_binary(self):
        _path = os.environ["PATH"]
        os.environ["PATH"] = "/does_not_exists"
        self._assertRaisesRegex(
            EnvironmentError,
            "nmap is not installed or could not be found in system path",
            NmapProcess,
            targets="127.0.0.1",
            options="-sP",
        )
        os.environ["PATH"] = _path

    def test_exec_env(self):
        self.assertRaises(
            EnvironmentError,
            NmapProcess,
            targets="127.0.0.1",
            options="-sV",
            fqp="/usr/bin/does-not-exists",
        )

    def test_exec(self):
        nmapobj = NmapProcess(targets="127.0.0.1", options="-sP")
        rc = nmapobj.run()
        parsed = NmapParser.parse(nmapobj.stdout)
        self.assertEqual(rc, 0)
        self.assertGreater(len(nmapobj.stdout), 0)
        self.assertIsInstance(parsed, NmapReport)

    def test_sudo_exec(self):
        nmapobj = NmapProcess(targets="127.0.0.1", options="-sP")
        self._assertRaisesRegex(
            EnvironmentError,
            "Username.*does not exists",
            nmapobj.sudo_run,
            run_as="non-existing-user",
        )
        self._assertRaisesRegex(
            EnvironmentError,
            "Username.*does not exists",
            nmapobj.sudo_run_background,
            run_as="non-existing-user",
        )

    def test_exec_reportsize(self):
        def make_nmproc_obj(targets, options):
            return NmapProcess(targets=targets, options=options)

        def start_all(nmprocs):
            for nmp in nmprocs:
                nmp.run()

        nb_targets = 20
        nm_target = "localhost"
        nm_opts = "-sT"

        nm_targets = [nm_target for i in range(nb_targets)]
        nm_procs = [make_nmproc_obj(t, nm_opts) for t in nm_targets]
        start_all(nm_procs)

        nm_procs = [make_nmproc_obj(t, nm_opts) for t in nm_targets]
        start_all(nm_procs)

        self.assertEqual(len(nm_procs), nb_targets)

        total_size = 0
        for i in range(len(nm_procs)):
            total_size += len(nm_procs[i].stdout)

        average_size = int(total_size / len(nm_procs))
        for nm in nm_procs:
            self.assertAlmostEqual(
                average_size, int(len(nm.stdout)), delta=200
            )


if __name__ == "__main__":
    test_suite = [
        "test_exec_env",
        "test_check_targets",
        "test_exec",
        "test_exec_reportsize",
    ]
    suite = unittest.TestSuite(map(TestNmapProcess, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-


class CPE(object):
    """
    CPE class offers an API for basic CPE objects.
    These objects could be found in NmapService or in <os> tag
    within NmapHost.

    :todo: interpret CPE string and provide appropriate API
    """

    def __init__(self, cpestring):
        self._cpestring = cpestring
        zk = [
            "cpe",
            "part",
            "vendor",
            "product",
            "version",
            "update",
            "edition",
            "language",
        ]
        self._cpedict = dict((k, "") for k in zk)
        splitup = cpestring.split(":")
        self._cpedict.update(dict(zip(zk, splitup)))

    @property
    def cpestring(self):
        """
        Accessor for the full CPE string.
        """
        return self._cpestring

    @property
    def cpedict(self):
        """
        Accessor for _cpedict
        """
        return self._cpedict

    def __repr__(self):
        return self._cpestring

    def get_part(self):
        """
        Returns the cpe part (/o, /h, /a)
        """
        return self._cpedict["part"]

    def get_vendor(self):
        """
        Returns the vendor name
        """
        return self._cpedict["vendor"]

    def get_product(self):
        """
        Returns the product name
        """
        return self._cpedict["product"]

    def get_version(self):
        """
        Returns the version of the cpe
        """
        return self._cpedict["version"]

    def get_update(self):
        """
        Returns the update version
        """
        return self._cpedict["update"]

    def get_edition(self):
        """
        Returns the cpe edition
        """
        return self._cpedict["edition"]

    def get_language(self):
        """
        Returns the cpe language
        """
        return self._cpedict["language"]

    def is_application(self):
        """
        Returns True if cpe describes an application
        """
        return self.get_part() == "/a"

    def is_hardware(self):
        """
        Returns True if cpe describes a hardware
        """
        return self.get_part() == "/h"

    def is_operating_system(self):
        """
        Returns True if cpe describes an operating system
        """
        return self.get_part() == "/o"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.diff import NmapDiffException

# sys.path.append("".join([os.path.dirname(__file__), "/../"]))
from libnmap.parser import NmapParser


class TestNmapParser(unittest.TestCase):
    def setUp(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        self.flist_full = [
            {"file": "%s/%s" % (fdir, "files/2_hosts.xml"), "hosts": 2},
            {"file": "%s/%s" % (fdir, "files/1_hosts.xml"), "hosts": 1},
            {
                "file": "%s/%s"
                % (fdir, "files/1_hosts_banner_ports_notsyn.xml"),
                "hosts": 1,
            },
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_banner_ports.xml"),
                "hosts": 1,
            },
            {"file": "%s/%s" % (fdir, "files/1_hosts_banner.xml"), "hosts": 1},
            {
                "file": "%s/%s" % (fdir, "files/2_hosts_version.xml"),
                "hosts": 2,
            },
            {"file": "%s/%s" % (fdir, "files/2_tcp_hosts.xml"), "hosts": 2},
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_nohostname.xml"),
                "hosts": 1,
            },
        ]

        self.flist_one = [
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_nohostname.xml"),
                "hosts": 1,
            }
        ]
        self.flist_two = [
            {
                "file": "%s/%s" % (fdir, "files/2_hosts.xml"),
                "hosts": 2,
                "elapsed": "134.36",
                "endtime": "1361738040",
                "summary": (
                    "Nmap done at Sun Feb 24 21:34:00 2013;"
                    " 2 IP addresses (2 hosts up) scanned"
                    " in 134.36 seconds"
                ),
            }
        ]

        self.hlist = [
            {"hostname": "localhost", "ports": 5, "open": 5},
            {"hostname": "localhost2", "ports": 4, "open": 2},
            {"hostname": "scanme.nmap.org", "ports": 4, "open": 3},
            {"hostname": "1.1.1.1", "ports": 2, "open": 0},
        ]
        self.flist_banner = [
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_banner.xml"),
                "banner": {
                    "631": "product: CUPS version: 1.4",
                    "3306": "product: MySQL version: 5.1.61",
                    "22": (
                        "product: OpenSSH version: 5.3"
                        " extrainfo: protocol 2.0"
                    ),
                    "25": (
                        "product: Postfix smtpd"
                        " hostname:  jambon.localdomain"
                    ),
                    "111": "",
                },
            }
        ]

        self.flist = self.flist_full

    def test_report_constructor(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            s = fd.read()
            fd.close()
            nr = NmapParser.parse(s)
            nr2 = NmapParser.parse(s)

            self.assertEqual(len(nr.hosts), testfile["hosts"])

            self.assertEqual(len(nr2.hosts), testfile["hosts"])
            self.assertEqual(
                sorted(nr2.get_raw_data()), sorted(nr.get_raw_data())
            )

    def test_get_ports(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            s = fd.read()
            fd.close()

            nr = NmapParser.parse(s)
            for h in nr.hosts:
                for th in self.hlist:
                    continue
                # TODO FIX THIS TEST
                #                    if th['hostname'] == h.hostnames[0]:
                #                    self.assertEqual(th['ports'], len(h.get_ports()))
                #                   self.assertEqual(th['open'], len(h.get_open_ports()))

                for np in h.get_open_ports():
                    sport = h.get_service(np[0], np[1])
                    self.assertEqual((sport.port, sport.protocol), np)

    def test_runstats(self):
        for testfile in self.flist_two:
            fd = open(testfile["file"], "r")
            s = fd.read()
            fd.close()
            nr = NmapParser.parse(s)
            self.assertEqual(getattr(nr, "endtime"), int(testfile["endtime"]))
            self.assertEqual(getattr(nr, "summary"), testfile["summary"])
            self.assertEqual(
                getattr(nr, "elapsed"), float(testfile["elapsed"])
            )

    def test_banner(self):
        for testfile in self.flist_banner:
            fd = open(testfile["file"], "r")
            nr = NmapParser.parse(fd.read())
            fd.close()

            for h in nr.hosts:
                for service in h.services:
                    b = service.banner
                    self.assertEqual(b, testfile["banner"][str(service.port)])

    def test_service_equal(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            np1 = NmapParser.parse(fd.read())
            fd.close()
            fd = open(testfile["file"], "r")
            np2 = NmapParser.parse(fd.read())
            fd.close()

            host1 = np1.hosts.pop()
            host2 = np2.hosts.pop()
            """All the service of the host must be compared and
               the hash should be also the same"""
            for i in range(len(host1.services)):
                self.assertEqual(
                    hash(host1.services[i]), hash(host2.services[i])
                )
                self.assertEqual(host1.services[i], host2.services[i])

            # print host1.serviceChanged(host2)

    def test_service_not_equal(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            np1 = NmapParser.parse(fd.read())
            fd.close()
            fd = open(testfile["file"], "r")
            np2 = NmapParser.parse(fd.read())
            fd.close()

            host1 = np1.hosts.pop()
            host2 = np2.hosts.pop()
            for i in range(len(host1.services)):
                host1.services[i]._state["state"] = "changed"
                self.assertNotEqual(host1.services[i], host2.services[i])
            # print "-----------"
            # print host1.serviceChanged(host2)
            # print "-----------"

    def test_host_not_equal(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            np1 = NmapParser.parse(fd.read())
            fd.close()
            fd = open(testfile["file"], "r")
            np2 = NmapParser.parse(fd.read())
            fd.close()

            host1 = np1.hosts.pop()
            host2 = np2.hosts.pop()

            host1.address = {"addr": "1.3.3.7", "addrtype": "ipv4"}
            self.assertNotEqual(host1, host2)

    def test_host_equal(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            np1 = NmapParser.parse(fd.read())
            fd.close()
            fd = open(testfile["file"], "r")
            np2 = NmapParser.parse(fd.read())
            fd.close()

            host1 = np1.hosts.pop()
            host2 = np2.hosts.pop()

            host1.services[0]._portid = "23"
            self.assertEqual(host1, host2)

    def test_host_address_changed(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        fd1 = open("%s/%s" % (fdir, "files/1_hosts_down.xml"), "r")
        fd2 = open("%s/%s" % (fdir, "files/1_hosts.xml"), "r")
        nr1 = NmapParser.parse(fd1.read())
        nr2 = NmapParser.parse(fd2.read())
        h1 = nr1.hosts[0]
        h2 = nr2.hosts[0]
        self.assertRaises(NmapDiffException, h1.diff, h2)

    def test_host_address_unchanged(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        fd1 = open("%s/%s" % (fdir, "files/1_hosts_down.xml"), "r")
        fd2 = open("%s/%s" % (fdir, "files/1_hosts.xml"), "r")
        fd3 = open("%s/%s" % (fdir, "files/1_hosts.xml"), "r")
        nr1 = NmapParser.parse(fd1.read())
        nr2 = NmapParser.parse(fd2.read())
        nr3 = NmapParser.parse(fd3.read())

        h1 = nr1.hosts.pop()
        h2 = nr2.hosts.pop()
        h3 = nr3.hosts.pop()

        self.assertRaises(NmapDiffException, h1.diff, h2)
        self.assertEqual(h2.diff(h3).changed(), set([]))
        self.assertEqual(h2.diff(h3).added(), set([]))
        self.assertEqual(h2.diff(h3).removed(), set([]))
        self.assertEqual(
            h2.diff(h3).unchanged(),
            set(
                [
                    "status",
                    "NmapService::tcp.22",
                    "NmapService::tcp.111",
                    "NmapService::tcp.631",
                    "hostnames",
                    "NmapService::tcp.3306",
                    "address",
                    "NmapService::tcp.25",
                    "mac_addr",
                ]
            ),
        )

    def test_diff_mac(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        host_ping = "{0}/files/1_host_ping.xml".format(fdir)
        host_ping_mac_changed = (
            "{0}/files/diff_1_host_ping_mac_changed.xml".format(fdir)
        )

        report_mac_original = NmapParser.parse_fromfile(host_ping)
        report_mac_changed = NmapParser.parse_fromfile(host_ping_mac_changed)

        report_diff = report_mac_original.diff(report_mac_changed)
        self.assertEqual(report_diff.changed(), set(["NmapHost::172.28.1.3"]))
        host_original = report_mac_original.hosts[0]
        host_mac_changed = report_mac_changed.hosts[0]
        host_diff = host_original.diff(host_mac_changed)
        self.assertEqual(host_diff.changed(), set(["mac_addr"]))


if __name__ == "__main__":
    test_suite = [
        "test_report_constructor",
        "test_get_ports",
        "test_runstats",
        "test_banner",
        "test_service_equal",
        "test_service_not_equal",
        "test_host_not_equal",
        "test_host_equal",
        "test_host_address_changed",
        "test_host_address_unchanged",
    ]

    suite = unittest.TestSuite(map(TestNmapParser, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.parser import NmapParser

rep = NmapParser.parse_fromfile("libnmap/test/files/os_scan6.xml")

print("{0}/{1} hosts up".format(rep.hosts_up, rep.hosts_total))
for _host in rep.hosts:
    if _host.is_up():
        print("{0} {1}".format(_host.address, " ".join(_host.hostnames)))
        if _host.os_fingerprinted:
            print("OS Fingerprint:")
            msg = ""
            for osm in _host.os.osmatches:
                print("Found Match:{0} ({1}%)".format(osm.name, osm.accuracy))
                for osc in osm.osclasses:
                    print("\tOS Class: {0}".format(osc.description))
                    for cpe in osc.cpelist:
                        print("\tCPE: {0}".format(cpe.cpestring))
        else:
            print("No fingerprint available")



#!/usr/bin/env python
from libnmap.parser import NmapParser, NmapParserException
from libnmap.process import NmapProcess

nm = NmapProcess("127.0.0.1", "-sP")
rc = nm.run()
if rc != 0:
    print("nmap scan failed: {0}".format(nm.stderr))

try:
    report = NmapParser.parse(nm.stdout)
except NmapParserException as e:
    print("Exception raised while parsing scan: {0}".format(e.msg))

print(len(nm.stdout))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.diff import NmapDiffException
from libnmap.parser import NmapParser

service1 = """
<port protocol="tcp" portid="22">
    <state state="open" reason="syn-ack" reason_ttl="0"/>
    <service name="ssh" method="table" conf="3"/>
</port>
"""

service2 = """
<port protocol="udp" portid="22">
    <state state="open" reason="syn-ack" reason_ttl="0"/>
    <service name="ssh" method="table" conf="3"/>
</port>
"""

service3 = """
<port protocol="udp" portid="53">
    <state state="open" reason="syn-ack" reason_ttl="0"/>
    <service name="ssh" method="table" conf="3"/>
</port>
"""

service4 = """
<port protocol="tcp" portid="22">
    <state state="closed" reason="syn-ack" reason_ttl="0"/>
    <service name="ssh" method="table" conf="3"/>
</port>
"""

service5 = """
<port protocol="tcp" portid="3306">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="mysql" product="MySQL" version="5.1.62"
            method="probed" conf="10"/>
</port>
"""

service6 = """
<port protocol="tcp" portid="3306">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="mysql" product="MySQL"
        version="5.1.61" method="probed" conf="10"/>
</port>
"""

service7 = """
<port protocol="tcp" portid="3306">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="mysql"
        product="MySQL" version="5.1.61" method="probed" conf="10"/>
</port>
"""

port_string = """
<port protocol="tcp" portid="25">
    <state state="filtered" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="smtp" method="table" conf="3"/>
</port>"""

port_string_other2 = """
<port protocol="tcp" portid="25">
    <state state="open" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="smtp" method="table" conf="3"/>
</port>"""

port_string_other3 = """
<port protocol="tcp" portid="22">
    <state state="open" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="ssh" method="table" conf="3"/>
</port>"""

port_string_other4 = """
<port protocol="tcp" portid="22">
    <state state="willy_woncka" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="willywoncka" method="table" conf="3"/>
</port>"""

port_string_other5 = """
<port protocol="tcp" portid="22">
    <state state="willy_woncka" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="ssh" method="table" conf="3"/>
</port>"""

port_string_other6 = """
<port protocol="tcp" portid="25">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="smtp" product="Postfix smtpd"
        hostname=" jambon.localdomain" method="probed" conf="10"/>
</port>"""

port_string_other7 = """
<port protocol="tcp" portid="111">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="rpcbind" method="probed" conf="10"/>
</port>"""

port_string_other8 = """
<port protocol="tcp" portid="631">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="ipp" product="CUPS" version="1.4"
        method="probed" conf="10"/>
</port>"""

port_string_other9 = """
<port protocol="tcp" portid="631">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="ipp" product="COPS" version="1.4"
        method="probed" conf="10"/>
</port>"""

port_string_other10 = """
<port protocol="tcp" portid="25">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="smtp" product="Postfix smtpd"
        hostname=" jambon.localdomain" method="probed" conf="10"/>
</port>"""

port_string_other11 = """
<port protocol="tcp" portid="25">
    <state state="open" reason="syn-ack" reason_ttl="69"/>
    <service name="smtp" product="Postfix smtpd"
        hostname=" jambon.localdomain" method="probed" conf="10"/>
</port>"""

port_string_other12 = """
<port protocol="tcp" portid="25">
    <state state="filtered" reason="admin-prohibited"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="smtp" method="table" conf="3"/>
</port>"""

port_string_other13 = """
<port protocol="tcp" portid="25">
    <state state="filtered" reason="patin"
        reason_ttl="253" reason_ip="109.133.192.1"/>
    <service name="smtp" method="table" conf="3"/>
</port>"""

port_noservice = """
<port protocol="udp" portid="3306">
<state state="closed" reason="port-unreach" reason_ttl="64" />
</port>"""

port_owner = """
<port protocol="tcp" portid="25">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="smtp" product="Postfix smtpd"
        hostname=" jambon.localdomain" method="probed" conf="10"/>
    <owner name="edwige"/>
</port>"""

port_tunnel = """
<port protocol="tcp" portid="443">
    <state state="open" reason="syn-ack" reason_ttl="64"/>
    <service name="https" method="probed" tunnel="ssl" conf="10"/>
</port>"""


class TestNmapService(unittest.TestCase):
    def setUp(self):
        self.fdir = os.path.dirname(os.path.realpath(__file__))
        self.s1 = NmapParser.parse(service1)
        self.s2 = NmapParser.parse(service2)
        self.s3 = NmapParser.parse(service3)
        self.s4 = NmapParser.parse(service4)
        self.s5 = NmapParser.parse(service5)
        self.s6 = NmapParser.parse(service6)
        self.s7 = NmapParser.parse(service7)

    def test_port_state_changed(self):
        nservice1 = NmapParser.parse(port_string)
        nservice2 = NmapParser.parse(port_string_other2)
        nservice3 = NmapParser.parse(port_string_other3)
        nservice4 = NmapParser.parse(port_string_other4)

        self.assertEqual(nservice1.diff(nservice2).changed(), set(["state"]))
        self.assertRaises(NmapDiffException, nservice1.diff, nservice3)
        self.assertRaises(NmapDiffException, nservice1.diff, nservice4)
        #
        self.assertRaises(NmapDiffException, nservice2.diff, nservice3)
        self.assertEqual(
            nservice3.diff(nservice4).changed(), set(["state", "service"])
        )

    def test_port_state_unchanged(self):
        nservice1 = NmapParser.parse(port_string)
        nservice2 = NmapParser.parse(port_string_other2)
        # nservice3 = NmapParser.parse(port_string_other3)
        # nservice4 = NmapParser.parse(port_string_other4)

        self.assertEqual(
            nservice1.diff(nservice2).unchanged(),
            set(["banner", "protocol", "port", "service", "id", "reason"]),
        )

    def test_port_service_changed(self):
        nservice1 = NmapParser.parse(port_string)
        nservice2 = NmapParser.parse(port_string_other2)
        nservice4 = NmapParser.parse(port_string_other4)
        nservice5 = NmapParser.parse(port_string_other5)
        nservice8 = NmapParser.parse(port_string_other8)
        nservice9 = NmapParser.parse(port_string_other9)

        self.assertEqual(nservice1.diff(nservice2).changed(), set(["state"]))
        self.assertEqual(nservice5.diff(nservice4).changed(), set(["service"]))
        # banner changed
        self.assertEqual(nservice8.diff(nservice9).changed(), set(["banner"]))

    def test_eq_service(self):
        self.assertNotEqual(NmapDiffException, self.s1, self.s2)
        self.assertNotEqual(self.s1, self.s3)
        self.assertNotEqual(self.s1, self.s4)

        self.assertNotEqual(self.s5, self.s6)
        self.assertEqual(self.s6, self.s7)

    def test_diff_service(self):
        self.assertRaises(NmapDiffException, self.s1.diff, self.s2)
        self.assertRaises(NmapDiffException, self.s1.diff, self.s3)
        self.assertEqual(self.s1.diff(self.s4).changed(), set(["state"]))
        self.assertEqual(
            self.s1.diff(self.s4).unchanged(),
            set(["banner", "protocol", "port", "service", "id", "reason"]),
        )

        self.assertEqual(self.s5.diff(self.s6).changed(), set(["banner"]))
        self.assertEqual(self.s6.diff(self.s6).changed(), set([]))

    def test_diff_reason(self):
        nservice12 = NmapParser.parse(port_string_other12)
        nservice13 = NmapParser.parse(port_string_other13)
        ddict = nservice12.diff(nservice13)
        self.assertEqual(ddict.changed(), set(["reason"]))

    def test_noservice(self):
        noservice = NmapParser.parse(port_noservice)
        self.assertEqual(noservice.service, "")

    def test_owner(self):
        serviceowner = NmapParser.parse(port_owner)
        self.assertEqual(serviceowner.owner, "edwige")

    def test_tunnel(self):
        servicetunnel = NmapParser.parse(port_tunnel)
        self.assertEqual(servicetunnel.tunnel, "ssl")

    def test_bannerdict(self):
        nmapreport = NmapParser.parse_fromfile(
            "{0}/files/dionaea_scan.xml".format(self.fdir)
        )
        dhttp = nmapreport.hosts[0].get_service(80)
        dftp = nmapreport.hosts[0].get_service(21)
        self.assertEqual(dhttp.banner_dict, {"product": "nginx"})
        self.assertEqual(
            dftp.banner_dict,
            {
                "product": "Synology DiskStation NAS ftpd",
                "devicetype": "storage-misc",
            },
        )


if __name__ == "__main__":
    test_suite = [
        "test_port_state_changed",
        "test_port_state_unchanged",
        "test_port_service_changed",
        "test_eq_service",
        "test_diff_service",
    ]
    suite = unittest.TestSuite(map(TestNmapService, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime

import pygeoip
from elasticsearch import Elasticsearch

from libnmap.parser import NmapParser


def store_report(nmap_report, database, index):
    rval = True
    for nmap_host in nmap_report.hosts:
        rv = store_reportitem(nmap_host, database, index)
        if rv is False:
            print(
                "Failed to store host {0} in "
                "elasticsearch".format(nmap_host.address)
            )
            rval = False

    return rval


def get_os(nmap_host):
    rval = {"vendor": "unknown", "product": "unknown"}
    if nmap_host.is_up() and nmap_host.os_fingerprinted:
        cpelist = nmap_host.os.os_cpelist()
        if len(cpelist):
            mcpe = cpelist.pop()
            rval.update(
                {"vendor": mcpe.get_vendor(), "product": mcpe.get_product()}
            )
    return rval


def get_geoip_code(address):
    gi = pygeoip.GeoIP("/usr/share/GeoIP/GeoIP.dat")
    return gi.country_code_by_addr(address)


def store_reportitem(nmap_host, database, index):
    host_keys = [
        "starttime",
        "endtime",
        "address",
        "hostnames",
        "ipv4",
        "ipv6",
        "mac",
        "status",
    ]
    jhost = {}
    for hkey in host_keys:
        if hkey == "starttime" or hkey == "endtime":
            val = getattr(nmap_host, hkey)
            jhost[hkey] = datetime.fromtimestamp(int(val) if len(val) else 0)
        else:
            jhost[hkey] = getattr(nmap_host, hkey)

    jhost.update({"country": get_geoip_code(nmap_host.address)})
    jhost.update(get_os(nmap_host))
    for nmap_service in nmap_host.services:
        reportitems = get_item(nmap_service)

        for ritem in reportitems:
            ritem.update(jhost)
            database.index(index=index, doc_type="NmapItem", body=ritem)
    return jhost


def get_item(nmap_service):
    service_keys = ["port", "protocol", "state"]
    ritems = []

    # create report item for basic port scan
    jservice = {}
    for skey in service_keys:
        jservice[skey] = getattr(nmap_service, skey)
    jservice["type"] = "port-scan"
    jservice["service"] = nmap_service.service
    jservice["service-data"] = nmap_service.banner
    ritems.append(jservice)

    # create report items from nse script output
    for nse_item in nmap_service.scripts_results:
        jnse = {}
        for skey in service_keys:
            jnse[skey] = getattr(nmap_service, skey)
        jnse["type"] = "nse-script"
        jnse["service"] = nse_item["id"]
        jnse["service-data"] = nse_item["output"]
        ritems.append(jnse)

    return ritems


xmlscans = [
    "../libnmap/test/files/1_hosts.xml",
    "../libnmap/test/files/full_sudo6.xml",
    "/vagrant/nmap_switches.xml",
    "/vagrant/nmap-5hosts.xml",
]

for xmlscan in xmlscans:
    nmap_report = NmapParser.parse_fromfile(xmlscan)

    if nmap_report:
        rep_date = datetime.fromtimestamp(int(nmap_report.started))
        index = "nmap-{0}".format(rep_date.strftime("%Y-%m-%d"))
        db = Elasticsearch()
        j = store_report(nmap_report, db, index)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from time import sleep

from libnmap.process import NmapProcess

nmap_proc = NmapProcess(targets="scanme.nmap.org", options="-sT")
nmap_proc.run_background()
while nmap_proc.is_running():
    print(
        "Nmap Scan running: ETC: {0} DONE: {1}%".format(
            nmap_proc.etc, nmap_proc.progress
        )
    )
    sleep(2)

print("rc: {0} output: {1}".format(nmap_proc.rc, nmap_proc.summary))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import unittest

from libnmap.parser import NmapParser, NmapParserException


class TestDefusedXML(unittest.TestCase):
    def setUp(self):
        if int(sys.version[0]) == 3:
            self._assertRaisesRegex = self.assertRaisesRegex
        else:
            self._assertRaisesRegex = self.assertRaisesRegexp

        self.billionlaugh = """<?xml version="1.0"?>
<!DOCTYPE lolz [
<!ENTITY lol "lol">
<!ELEMENT lolz (#PCDATA)>
<!ENTITY lol1 "&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;">
<!ENTITY lol2 "&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;">
<!ENTITY lol3 "&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;">
<!ENTITY lol4 "&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;">
<!ENTITY lol5 "&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;">
<!ENTITY lol6 "&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;">
<!ENTITY lol7 "&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;">
<!ENTITY lol8 "&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;">
<!ENTITY lol9 "&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;">
]>
<lolz>&lol9;</lolz>
        """
        self.fdir = os.path.dirname(os.path.realpath(__file__))
        self.billionlaugh_file = "{0}/files/{1}".format(
            self.fdir, "billion_laugh.xml"
        )
        self.external_entities_file = "{0}/files/{1}".format(
            self.fdir, "defused_et_local_includer.xml"
        )

    def test_billion_laugh(self):
        self._assertRaisesRegex(
            NmapParserException,
            ".*EntitiesForbidden",
            NmapParser.parse_fromstring,
            self.billionlaugh,
        )

    def test_external_entities(self):
        self._assertRaisesRegex(
            NmapParserException,
            ".*EntitiesForbidden",
            NmapParser.parse_fromfile,
            self.external_entities_file,
        )


if __name__ == "__main__":
    # test_suite = ["test_external_entities"]
    test_suite = ["test_billion_laugh", "test_external_entities"]
    suite = unittest.TestSuite(map(TestDefusedXML, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.parser import NmapParser

rep1 = NmapParser.parse_fromfile("libnmap/test/files/1_hosts.xml")
rep2 = NmapParser.parse_fromfile("libnmap/test/files/1_hosts_diff.xml")

rep1_items_changed = rep1.diff(rep2).changed()
changed_host_id = rep1_items_changed.pop().split("::")[1]

changed_host1 = rep1.get_host_byid(changed_host_id)
changed_host2 = rep2.get_host_byid(changed_host_id)
host1_items_changed = changed_host1.diff(changed_host2).changed()

changed_service_id = host1_items_changed.pop().split("::")[1]
changed_service1 = changed_host1.get_service_byid(changed_service_id)
changed_service2 = changed_host2.get_service_byid(changed_service_id)
service1_items_changed = changed_service1.diff(changed_service2).changed()

for diff_attr in service1_items_changed:
    print(
        "diff({0}, {1}) [{2}:{3}] [{4}:{5}]".format(
            changed_service1.id,
            changed_service2.id,
            diff_attr,
            getattr(changed_service1, diff_attr),
            diff_attr,
            getattr(changed_service2, diff_attr),
        )
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.parser import NmapParser

rep = NmapParser.parse_fromfile("libnmap/test/files/full_sudo6.xml")

print(
    "Nmap scan discovered {0}/{1} hosts up".format(
        rep.hosts_up, rep.hosts_total
    )
)
for _host in rep.hosts:
    if _host.is_up():
        print(
            "+ Host: {0} {1}".format(_host.address, " ".join(_host.hostnames))
        )

        # get CPE from service if available
        for s in _host.services:
            print(
                "    Service: {0}/{1} ({2})".format(
                    s.port, s.protocol, s.state
                )
            )
            # NmapService.cpelist returns an array of CPE objects
            for _serv_cpe in s.cpelist:
                print("        CPE: {0}".format(_serv_cpe.cpestring))

        if _host.os_fingerprinted:
            print("  OS Fingerprints")
            for osm in _host.os.osmatches:
                print(
                    "    Found Match:{0} ({1}%)".format(osm.name, osm.accuracy)
                )
                # NmapOSMatch.get_cpe() method return an array of string
                # unlike NmapOSClass.cpelist which returns an array of CPE obj
                for cpe in osm.get_cpe():
                    print("\t    CPE: {0}".format(cpe))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from libnmap.objects.os import CPE


class TestNmapFP(unittest.TestCase):
    def setUp(self):
        self.cpelist = [
            "cpe:/a:apache:http_server:2.2.22",
            "cpe:/a:heimdal:kerberos",
            "cpe:/a:openbsd:openssh:5.9p1",
            "cpe:/o:apple:iphone_os:5",
            "cpe:/o:apple:mac_os_x:10.8",
            "cpe:/o:apple:mac_os_x",
            "cpe:/o:linux:linux_kernel:2.6.13",
            "cpe:/o:linux:linux_kernel",
            "cpe:/o:microsoft:windows_7",
            "cpe:/o:microsoft:windows_7::-:professional",
            "cpe:/o:microsoft:windows_7::sp1",
            "cpe:/o:microsoft:windows",
            "cpe:/o:microsoft:windows_server_2008::beta3",
            "cpe:/o:microsoft:windows_server_2008",
            "cpe:/o:microsoft:windows_server_2008::sp1",
            "cpe:/o:microsoft:windows_vista::-",
            "cpe:/o:microsoft:windows_vista::sp1",
            "cpe:/o:microsoft:windows_vista::sp2",
        ]

    def test_cpe(self):
        apa = CPE(self.cpelist[0])

        self.assertTrue(apa.is_application())
        self.assertFalse(apa.is_hardware())
        self.assertFalse(apa.is_operating_system())
        self.assertEqual(
            apa.cpedict,
            {
                "cpe": "cpe",
                "edition": "",
                "language": "",
                "part": "/a",
                "product": "http_server",
                "update": "",
                "vendor": "apache",
                "version": "2.2.22",
            },
        )

        win = CPE(self.cpelist[12])
        self.assertEqual(win.get_vendor(), "microsoft")
        self.assertEqual(win.get_product(), "windows_server_2008")
        self.assertEqual(win.get_version(), "")
        self.assertEqual(win.get_update(), "beta3")
        self.assertEqual(win.get_edition(), "")
        self.assertEqual(win.get_language(), "")

    def test_full_cpe(self):
        cpestr = "cpe:/a:mozilla:firefox:2.0::osx:es-es"
        resdict = {
            "part": "/a",
            "vendor": "mozilla",
            "product": "firefox",
            "version": "2.0",
            "update": "",
            "edition": "osx",
            "language": "es-es",
        }
        ocpe = CPE(cpestr)
        objdict = {
            "part": ocpe.get_part(),
            "vendor": ocpe.get_vendor(),
            "product": ocpe.get_product(),
            "version": ocpe.get_version(),
            "update": ocpe.get_update(),
            "language": ocpe.get_language(),
            "edition": ocpe.get_edition(),
        }
        self.assertEqual(objdict, resdict)
        # self.assertEqual(ocpe.cpedict, resdict)
        self.assertEqual(str(ocpe), cpestr)


if __name__ == "__main__":
    test_suite = ["test_cpe", "test_full_cpe"]
    suite = unittest.TestSuite(map(TestNmapFP, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
import json

from bson.objectid import ObjectId
from pymongo import MongoClient

from libnmap.parser import NmapParser
from libnmap.plugins.backendplugin import NmapBackendPlugin
from libnmap.reportjson import ReportEncoder


class NmapMongodbPlugin(NmapBackendPlugin):
    """
    This class handle the persistence of NmapRepport object in mongodb
    Implementation is made using pymongo
    Object of this class must be create via the
    BackendPluginFactory.create(**url) where url is a named dict like
    {'plugin_name': "mongodb"} this dict may receive all the param
    MongoClient() support
    """

    def __init__(self, dbname=None, store=None, **kwargs):
        NmapBackendPlugin.__init__(self)
        if dbname is not None:
            self.dbname = dbname
        if store is not None:
            self.store = store
        self.dbclient = MongoClient(**kwargs)
        self.collection = self.dbclient[self.dbname][self.store]

    def insert(self, report):
        """
        create a json object from an NmapReport instance
        :param NmapReport: obj to insert
        :return: str id
        """
        j = json.dumps(report, cls=ReportEncoder)
        try:
            oid = self.collection.insert(json.loads(j))
        except Exception as e:
            em = "Failed to insert nmap object in MongoDB: {0}".format(e)
            raise Exception(em)
        return str(oid)

    def get(self, str_report_id=None):
        """select a NmapReport by Id
        :param str: id
        :return: NmapReport object
        """
        rid = str_report_id
        nmapreport = None
        if str_report_id is not None and isinstance(str_report_id, str):
            rid = ObjectId(str_report_id)

        if isinstance(rid, ObjectId):
            # get a specific report by mongo's id
            resultset = self.collection.find({"_id": rid})
            if resultset.count() == 1:
                # search by id means only one in the iterator
                record = resultset[0]
                # remove mongo's id to recreate the NmapReport Obj
                del record["_id"]
                nmapreport = NmapParser.parse_fromdict(record)
        return nmapreport

    def getall(self, dict_filter=None):
        """return a list of tuple (id,NmapReport) saved in the backend
        TODO : add a filter capability
        """
        nmapreportlist = []
        resultset = self.collection.find()
        for report in resultset:
            oid = report["_id"]
            del report["_id"]
            nmapreport = NmapParser.parse_fromdict(report)
            nmapreportlist.append((oid, nmapreport))
        return nmapreportlist

    def delete(self, report_id=None):
        """
        delete an obj from the backend
        :param str: id
        :return: dict document with result or None
        """
        if report_id is not None and isinstance(report_id, str):
            return self.collection.remove({"_id": ObjectId(report_id)})
        else:
            return self.collection.remove({"_id": report_id})



#!/usr/bin/env python
import inspect
import sys


class BackendPluginFactory(object):
    """
    This is a backend plugin factory a backend instance MUST be
    created via the static method create()
    ie : mybackend = BackendPluginFactory.create()
    """

    @classmethod
    def create(cls, plugin_name="mongodb", **kwargs):
        """Import the needed lib and return an object NmapBackendPlugin
        representing the backend of your desire.
        NmapBackendPlugin is an abstract class, to know what argument
        need to be given, review the code of the subclass you need
        :param plugin_name: str : name of the py file without .py
        :return: NmapBackend (abstract class on top of all plugin)
        """
        backendplugin = None
        plugin_path = "libnmap.plugins.{0}".format(plugin_name)
        __import__(plugin_path)
        pluginobj = sys.modules[plugin_path]
        pluginclasses = inspect.getmembers(pluginobj, inspect.isclass)
        for classname, classobj in pluginclasses:
            if inspect.getmodule(classobj).__name__.find(plugin_path) == 0:
                try:
                    backendplugin = classobj(**kwargs)
                except Exception as error:
                    raise Exception(
                        "Cannot create Backend {0}: {1}".format(
                            classname, error
                        )
                    )
        return backendplugin



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.process import NmapProcess

nmap_proc = NmapProcess(targets="scanme.nmap.org", options="-sV")
nmap_proc.run_background()
while nmap_proc.is_running():
    nmaptask = nmap_proc.current_task
    if nmaptask:
        print(
            "Task {0} ({1}): ETC: {2} DONE: {3}%".format(
                nmaptask.name, nmaptask.status, nmaptask.etc, nmaptask.progress
            )
        )
print("rc: {0} output: {1}".format(nmap_proc.rc, nmap_proc.summary))
print(nmap_proc.stdout)
print(nmap_proc.stderr)



# -*- coding: utf-8 -*-

import json
from datetime import datetime

from elasticsearch import Elasticsearch

from libnmap.plugins.backendplugin import NmapBackendPlugin
from libnmap.reportjson import ReportEncoder


class NmapElasticsearchPlugin(NmapBackendPlugin):
    """
        This class enables the user to store and manipulate nmap reports \
        in a elastic search db.
    """

    def __init__(self, index=None):
        if index is None:
            self.index = "nmap.{0}".format(datetime.now().strftime("%Y-%m-%d"))
        else:
            self.index = index
        self._esapi = Elasticsearch()

    def insert(self, report, doc_type=None):
        """
        insert NmapReport in the backend
        :param NmapReport:
        :return: str the ident of the object in the backend for
        future usage
        or None
        """
        if doc_type is None:
            doc_type = "NmapReport"
        j = json.dumps(report, cls=ReportEncoder)
        res = self._esapi.index(
            index=self.index, doc_type=doc_type, body=json.loads(j)
        )
        rc = res["_id"]
        return rc

    def delete(self, id):
        """
        delete NmapReport if the backend
        :param id: str
        """
        raise NotImplementedError

    def get(self, id):
        """
        retrieve a NmapReport from the backend
        :param id: str
        :return: NmapReport
        """
        res = self._esapi.get(index=self.index, doc_type="NmapReport", id=id)
        rc = res["_source"]
        return rc

    def getall(self, filter=None):
        """
        :return: collection of tuple (id,NmapReport)
        :param filter: Nice to have implement a filter capability
        """
        rsearch = self._esapi.search(
            index=self.index, body={"query": {"match_all": {}}}
        )
        print("--------------------")
        print(type(rsearch))
        print(rsearch)
        print("------------")



#!/usr/bin/env python

from time import sleep

from libnmap.process import NmapProcess


def make_nmproc_obj(targets, options):
    return NmapProcess(targets=targets, options=options)


def start_all_bg(nmprocs):
    for nmp in nmprocs:
        nmp.run_background()


def any_running(nmprocs):
    return any([nmp.is_running() for nmp in nmprocs])


def summarize(nmprocs):
    for nmp in nmprocs:
        print("rc: {0} output: {1}".format(nmp.rc, len(nmp.stdout)))
    print(nmp.stdout)


nb_targets = 10
nm_target = "localhost"
nm_opts = "-sP"

nm_targets = [nm_target for i in range(nb_targets)]
nm_procs = [make_nmproc_obj(t, nm_opts) for t in nm_targets]
start_all_bg(nm_procs)

while any_running(nm_procs):
    sleep(5)

summarize(nm_procs)



#!/usr/bin/env python

import json
from datetime import datetime

from libnmap.parser import NmapParser
from libnmap.plugins.es import NmapElasticsearchPlugin
from libnmap.reportjson import ReportDecoder

nmap_report = NmapParser.parse_fromfile("libnmap/test/files/1_hosts.xml")
mindex = datetime.fromtimestamp(nmap_report.started).strftime("%Y-%m-%d")
db = NmapElasticsearchPlugin(index=mindex)
dbid = db.insert(nmap_report)
nmap_json = db.get(dbid)

nmap_obj = json.loads(json.dumps(nmap_json), cls=ReportDecoder)
print(nmap_obj)
# print(db.getall())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.parser import NmapParser
from libnmap.plugins.backendplugin import NmapBackendPlugin
from libnmap.plugins.backendpluginFactory import BackendPluginFactory


class TestNmapBackendPlugin(unittest.TestCase):
    """
    This testing class will tests each plugins
    The following test need to be done :
       - test the factory
       - test all the method of the class NmapBackendPlugin:
          - Verify implmented/notImplemented
          - Verify the behaviour (ie insert must insert)
    To support a new plugin or a new way to instanciate a plugin, add a dict
    with the necessary parameter in the urls table define in setUp
    All testcase must loop thru theses urls to validate a plugins
    """

    def setUp(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        self.flist_full = [
            {"file": "{0}/{1}".format(fdir, "files/2_hosts.xml"), "hosts": 2},
            {"file": "{0}/{1}".format(fdir, "files/1_hosts.xml"), "hosts": 1},
            {
                "file": "{0}/{1}".format(
                    fdir, "files/1_hosts_banner_ports_notsyn.xml"
                ),
                "hosts": 1,
            },
            {
                "file": "{0}/{1}".format(
                    fdir, "files/1_hosts_banner_ports.xml"
                ),
                "hosts": 1,
            },
            {
                "file": "{0}/{1}".format(fdir, "files/1_hosts_banner.xml"),
                "hosts": 1,
            },
            {
                "file": "{0}/{1}".format(fdir, "files/2_hosts_version.xml"),
                "hosts": 2,
            },
            {
                "file": "{0}/{1}".format(fdir, "files/2_tcp_hosts.xml"),
                "hosts": 2,
            },
            {
                "file": "{0}/{1}".format(fdir, "files/1_hosts_nohostname.xml"),
                "hosts": 1,
            },
        ]
        self.flist = self.flist_full
        # build a list of NmapReport
        self.reportList = []
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            s = fd.read()
            fd.close()
            nrp = NmapParser.parse(s)
            self.reportList.append(nrp)

        self.urls = [
            {"plugin_name": "mongodb"},
            {
                "plugin_name": "sql",
                "url": "sqlite:////tmp/reportdb.sql",
                "echo": False,
            },
            {
                "plugin_name": "sql",
                "url": "mysql+pymysql://root@localhost/poulet",
                "echo": False,
            },
        ]

    def test_backend_factory(self):
        """test_factory BackendPluginFactory.create(**url)
        Invoke factory and test that the object is of the right classes
        """
        for url in self.urls:
            backend = BackendPluginFactory.create(**url)
            self.assertEqual(isinstance(backend, NmapBackendPlugin), True)
            className = "Nmap%sPlugin" % url["plugin_name"].title()
            self.assertEqual(backend.__class__.__name__, className, True)

    def test_backend_insert(self):
        """test_insert
        best way to insert is to call save() of nmapreport :P
        """
        for nrp in self.reportList:
            for url in self.urls:
                # create the backend factory object
                backend = BackendPluginFactory.create(**url)
                # save the report
                returncode = nrp.save(backend)
                # test return code
                self.assertNotEqual(returncode, None)

    def test_backend_get(self):
        """test_backend_get
        inset all report and save the returned id in a list
        then get each id and create a new list of report
        compare each report (assume eq)
        """
        id_list = []
        result_list = []
        for url in self.urls:
            backend = BackendPluginFactory.create(**url)
            for nrp in self.reportList:
                id_list.append(nrp.save(backend))
            for rep_id in id_list:
                result_list.append(backend.get(rep_id))
            self.assertEqual(len(result_list), len(self.reportList))
            self.assertEqual((result_list), (self.reportList))
            id_list = []
            result_list = []

    def test_backend_getall(self):
        pass

    def test_backend_delete(self):
        """test_backend_delete
        inset all report and save the returned id in a list
        for each id remove the item and test if not present
        """
        id_list = []
        result_list = []
        for url in self.urls:
            backend = BackendPluginFactory.create(**url)
            for nrp in self.reportList:
                id_list.append(nrp.save(backend))
            for rep_id in id_list:
                result_list.append(backend.delete(rep_id))
                self.assertEqual(backend.get(rep_id), None)
            id_list = []
            result_list = []


if __name__ == "__main__":
    test_suite = [
        "test_backend_factory",
        "test_backend_insert",
        "test_backend_get",
        "test_backend_getall",
        "test_backend_delete",
    ]
    suite = unittest.TestSuite(map(TestNmapBackendPlugin, test_suite))
    test_result = unittest.TextTestRunner(verbosity=5).run(suite)



#!/usr/bin/env python
import json
from datetime import datetime

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.schema import Column
from sqlalchemy.types import DateTime, Integer, LargeBinary

from libnmap.plugins.backendplugin import NmapBackendPlugin
from libnmap.reportjson import ReportDecoder, ReportEncoder

Base = declarative_base()


class NmapSqlPlugin(NmapBackendPlugin):
    """
    This class handle the persistence of NmapRepport object in SQL backend
    Implementation is made using sqlalchemy(0.8.1)
    usage :

    #get a nmapReport object
    from libnmap.parser import NmapParser
    from libnmap.reportjson import ReportDecoder, ReportEncoder
    import json
    nmap_report_obj = NmapParser.parse_fromfile(
           '/home/vagrant/python-nmap-lib/libnmap/test/files/1_hosts.xml')

     #get a backend with in memory sqlite
     from libnmap.plugins.backendpluginFactory import BackendPluginFactory
     mybackend_mem = BackendPluginFactory.create(plugin_name='sql',
                                                 url='sqlite://',
                                                 echo=True)

     mybackend_mysql = BackendPluginFactory.create(plugin_name='sql',
                        url='mysql+mysqldb://scott:tiger@localhost/foo',
                        echo=True)
     mybackend = BackendPluginFactory.create(plugin_name='sql',
                                    url='sqlite:////tmp/reportdb.sql',
                                    echo=True)
     #lets save!!
     nmap_report_obj.save(mybackend)
     mybackend.getall()
     mybackend.get(1)
    """

    class Reports(Base):
        """
        Embeded class for ORM map NmapReport to a
        simple three column table
        """

        __tablename__ = "reports"

        id = Column("report_id", Integer, primary_key=True)
        inserted = Column("inserted", DateTime(), default="now")
        report_json = Column("report_json", LargeBinary())

        def __init__(self, obj_NmapReport):
            self.inserted = datetime.fromtimestamp(obj_NmapReport.endtime)
            dumped_json = json.dumps(obj_NmapReport, cls=ReportEncoder)
            self.report_json = bytes(dumped_json.encode("UTF-8"))

        def decode(self):
            json_decoded = self.report_json.decode("utf-8")
            nmap_report_obj = json.loads(json_decoded, cls=ReportDecoder)
            return nmap_report_obj

    def __init__(self, **kwargs):
        """
        constructor receive a **kwargs as the **kwargs in the sqlalchemy
        create_engine() method (see sqlalchemy docs)
        You must add to this **kwargs an 'url' key with the url to your
        database
        This constructor will :
        - create all the necessary obj to discuss with the DB
        - create all the mapping(ORM)

        todo : support the : sqlalchemy.engine_from_config

        :param **kwargs:
        :raises: ValueError if no url is given,
                all exception sqlalchemy can throw
        ie sqlite in memory url='sqlite://' echo=True
        ie sqlite file on hd url='sqlite:////tmp/reportdb.sql' echo=True
        ie mysql url='mysql+mysqldb://scott:tiger@localhost/foo'
        """
        NmapBackendPlugin.__init__(self)
        self.engine = None
        self.url = None
        self.Session = sessionmaker()

        if "url" not in kwargs:
            raise ValueError
        self.url = kwargs["url"]
        del kwargs["url"]
        try:
            self.engine = create_engine(self.url, **kwargs)
            Base.metadata.create_all(bind=self.engine, checkfirst=True)
            self.Session.configure(bind=self.engine)
        except Exception as e:
            raise (e)

    def insert(self, nmap_report):
        """
           insert NmapReport in the backend

           :param NmapReport:

           :returns: the ident of the object in the backend for future usage \
           or None
        """
        sess = self.Session()
        report = NmapSqlPlugin.Reports(nmap_report)
        sess.add(report)
        sess.commit()
        reportid = report.id
        sess.close()
        return reportid if reportid else None

    def get(self, report_id=None):
        """
        retrieve a NmapReport from the backend

        :param id: str

        :returns: NmapReport
        """
        if report_id is None:
            raise ValueError
        sess = self.Session()
        orp = sess.query(NmapSqlPlugin.Reports).filter_by(id=report_id)
        our_report = orp.first()
        sess.close()
        return our_report.decode() if our_report else None

    def getall(self):
        """
        :param filter: Nice to have implement a filter capability

        :returns: collection of tuple (id,NmapReport)
        """
        sess = self.Session()
        nmapreportList = []
        for report in sess.query(NmapSqlPlugin.Reports).order_by(
            NmapSqlPlugin.Reports.inserted
        ):
            nmapreportList.append((report.id, report.decode()))
        sess.close()
        return nmapreportList

    def delete(self, report_id=None):
        """
        Remove a report from the backend

        :param id: str

        :returns: The number of rows deleted
        """
        if report_id is None:
            raise ValueError
        nb_line = 0
        sess = self.Session()
        rpt = sess.query(NmapSqlPlugin.Reports).filter_by(id=report_id)
        nb_line = rpt.delete()
        sess.commit()
        sess.close()
        return nb_line



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.parser import NmapParser


def nested_obj(objname):
    rval = None
    splitted = objname.split("::")
    if len(splitted) == 2:
        rval = splitted
    return rval


def print_diff_added(obj1, obj2, added):
    for akey in added:
        nested = nested_obj(akey)
        if nested is not None:
            if nested[0] == "NmapHost":
                subobj1 = obj1.get_host_byid(nested[1])
            elif nested[0] == "NmapService":
                subobj1 = obj1.get_service_byid(nested[1])
            print("+ {0}".format(subobj1))
        else:
            print("+ {0} {1}: {2}".format(obj1, akey, getattr(obj1, akey)))


def print_diff_removed(obj1, obj2, removed):
    for rkey in removed:
        nested = nested_obj(rkey)
        if nested is not None:
            if nested[0] == "NmapHost":
                subobj2 = obj2.get_host_byid(nested[1])
            elif nested[0] == "NmapService":
                subobj2 = obj2.get_service_byid(nested[1])
            print("- {0}".format(subobj2))
        else:
            print("- {0} {1}: {2}".format(obj2, rkey, getattr(obj2, rkey)))


def print_diff_changed(obj1, obj2, changes):
    for mkey in changes:
        nested = nested_obj(mkey)
        if nested is not None:
            if nested[0] == "NmapHost":
                subobj1 = obj1.get_host_byid(nested[1])
                subobj2 = obj2.get_host_byid(nested[1])
            elif nested[0] == "NmapService":
                subobj1 = obj1.get_service_byid(nested[1])
                subobj2 = obj2.get_service_byid(nested[1])
            print_diff(subobj1, subobj2)
        else:
            print(
                "~ {0} {1}: {2} => {3}".format(
                    obj1, mkey, getattr(obj2, mkey), getattr(obj1, mkey)
                )
            )


def print_diff(obj1, obj2):
    ndiff = obj1.diff(obj2)

    print_diff_changed(obj1, obj2, ndiff.changed())
    print_diff_added(obj1, obj2, ndiff.added())
    print_diff_removed(obj1, obj2, ndiff.removed())


def main():
    newrep = NmapParser.parse_fromfile(
        "libnmap/test/files/2_hosts_achange.xml"
    )
    oldrep = NmapParser.parse_fromfile("libnmap/test/files/1_hosts.xml")

    print_diff(newrep, oldrep)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json

from libnmap.parser import NmapParser
from libnmap.reportjson import ReportDecoder, ReportEncoder

nmap_report_obj = NmapParser.parse_fromfile("libnmap/test/files/1_hosts.xml")

# create a json object from an NmapReport instance
nmap_report_json = json.dumps(nmap_report_obj, cls=ReportEncoder)
print(nmap_report_json)
# create a NmapReport instance from a json object
nmap_report_obj = json.loads(nmap_report_json, cls=ReportDecoder)
print(nmap_report_obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.parser import NmapParser, NmapParserException
from libnmap.process import NmapProcess


# start a new nmap scan on localhost with some specific options
def do_scan(targets, options):
    parsed = None
    nmproc = NmapProcess(targets, options)
    rc = nmproc.run()
    if rc != 0:
        print("nmap scan failed: {0}".format(nmproc.stderr))

    try:
        parsed = NmapParser.parse(nmproc.stdout)
    except NmapParserException as e:
        print("Exception raised while parsing scan: {0}".format(e.msg))

    return parsed


# print scan results from a nmap report
def print_scan(nmap_report):
    print(
        "Starting Nmap {0} ( http://nmap.org ) at {1}".format(
            nmap_report.version, nmap_report.started
        )
    )

    for host in nmap_report.hosts:
        if len(host.hostnames):
            tmp_host = host.hostnames.pop()
        else:
            tmp_host = host.address

        print("Nmap scan report for {0} ({1})".format(tmp_host, host.address))
        print("Host is {0}.".format(host.status))
        print("  PORT     STATE         SERVICE")

        for serv in host.services:
            pserv = "{0:>5s}/{1:3s}  {2:12s}  {3}".format(
                str(serv.port), serv.protocol, serv.state, serv.service
            )
            if len(serv.banner):
                pserv += " ({0})".format(serv.banner)
            print(pserv)
    print(nmap_report.summary)


if __name__ == "__main__":
    report = do_scan("127.0.0.1", "-sV")
    if report:
        print_scan(report)
    else:
        print("No results returned")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os
import unittest

from libnmap.parser import NmapParser
from libnmap.reportjson import ReportDecoder, ReportEncoder


class TestReportJson(unittest.TestCase):
    def setUp(self):
        self.fdir = os.path.dirname(os.path.realpath(__file__))
        self.xml_ref_file = "{0}/files/2_hosts.xml".format(self.fdir)
        self.json_ref_file = "{0}/files/2_hosts.json".format(self.fdir)

    def test_reportencode(self):
        nmap_report_obj = NmapParser.parse_fromfile(self.xml_ref_file)
        nmap_report_json = json.loads(
            json.dumps(nmap_report_obj, cls=ReportEncoder)
        )
        with open(self.json_ref_file, "r") as fd:
            nmap_report_json_ref = json.load(fd)
            self.assertEqual(nmap_report_json_ref, nmap_report_json)

    def test_reportdecode(self):
        nmap_report_obj_ref = NmapParser.parse_fromfile(self.xml_ref_file)

        with open(self.json_ref_file, "r") as fd:
            nmap_report_json_ref = json.dumps(json.load(fd))
            nmap_report_obj = json.loads(
                nmap_report_json_ref, cls=ReportDecoder
            )
            self.assertEqual(nmap_report_obj_ref, nmap_report_obj)


if __name__ == "__main__":
    test_suite = ["test_reportencode", "test_reportdecode"]
    suite = unittest.TestSuite(map(TestReportJson, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.parser import NmapParser, NmapParserException


class TestNmapParser(unittest.TestCase):
    def setUp(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        self.flist_full = [
            {"file": "%s/%s" % (fdir, "files/2_hosts.xml"), "hosts": 2},
            {"file": "%s/%s" % (fdir, "files/1_hosts.xml"), "hosts": 1},
            {
                "file": "%s/%s"
                % (fdir, "files/1_hosts_banner_ports_notsyn.xml"),
                "hosts": 1,
            },
            # {'file': "%s/%s" % (fdir,
            #                      'files/1_hosts_banner_ports_xmas.xml'),
            #                      'hosts': 1},
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_banner_ports.xml"),
                "hosts": 1,
            },
            {"file": "%s/%s" % (fdir, "files/1_hosts_banner.xml"), "hosts": 1},
            {
                "file": "%s/%s" % (fdir, "files/2_hosts_version.xml"),
                "hosts": 2,
            },
            # {'file': "%s/%s" % (fdir,
            #                      'files/2_null_hosts.xml'), 'hosts': 2},
            {"file": "%s/%s" % (fdir, "files/2_tcp_hosts.xml"), "hosts": 2},
            {
                "file": "%s/%s" % (fdir, "files/1_hosts_nohostname.xml"),
                "hosts": 1,
            },
        ]
        self.flist = self.flist_full

        self.ports_string = """<ports><extraports state="closed" count="996">
        <extrareasons reason="resets" count="996"/>
        </extraports>
        <port protocol="tcp" portid="22">
            <state state="open" reason="syn-ack" reason_ttl="53"/>
            <service name="ssh" method="table" conf="3"/>
        </port>
        <port protocol="tcp" portid="25">
            <state state="filtered" reason="admin-prohibited" \
                    reason_ttl="253" reason_ip="109.133.192.1"/>
        <service name="smtp" method="table" conf="3"/>
        </port>
        <port protocol="tcp" portid="80">
            <state state="open" reason="syn-ack" reason_ttl="51"/>
            <service name="http" method="table" conf="3"/>
        </port>
        <port protocol="tcp" portid="9929">
            <state state="open" reason="syn-ack" reason_ttl="53"/>
            <service name="nping-echo" method="table" conf="3"/>
        </port>
        </ports>"""

        self.ports_string2 = """<ports><extraports state="closed" count="996">
        <extrareasons reason="resets" count="996"/>
        </extraports>
        <port protocol="tcp" portid="A2">
            <state state="open" reason="syn-ack" reason_ttl="53"/>
            <service name="ssh" method="table" conf="3"/>
        </port>
        <port protocol="tcp" portid="25">
            <state state="filtered" reason="admin-prohibited" \
                    reason_ttl="253" reason_ip="109.133.192.1"/>
            <service name="smtp" method="table" conf="3"/>
        </port>
        <port protocol="tcp" portid="80">
            <state state="open" reason="syn-ack" reason_ttl="51"/>i
            <service name="http" method="table" conf="3"/></port>
        <port protocol="tcp" portid="9929">
            <state state="open" reason="syn-ack" reason_ttl="53"/>
            <service name="nping-echo" method="table" conf="3"/>
        </port>
        </ports>"""

        self.port_string = """
        <port protocol="tcp" portid="25">
        <state state="filtered" reason="admin-prohibited"
                reason_ttl="253" reason_ip="109.133.192.1"/>
        <service name="smtp" method="table" conf="3"/>
        </port>"""

        self.port_string2 = """
        <port protocol="tcp" portid="">
            <state state="filtered" reason="admin-prohibited"
                    reason_ttl="253" reason_ip="109.133.192.1"/>
            <service name="smtp" method="table" conf="3"/>
        </port>"""

        self.port_string3 = "<port></port>"
        self.port_string4 = ""
        self.port_string5 = "GINGERBREADMAN"
        self.port_string6 = """
        <port protocol="tcp" portid="FOOL">
        <state state="filtered" reason="admin-prohibited"
                reason_ttl="253" reason_ip="109.133.192.1"/>
        <service name="smtp" method="table" conf="3"/>
        </port>"""

        self.port_string7 = """
        <port protocol="tcp" portid="22">
        <stAAte state="filtered" reason="admin-prohibited"
                reason_ttl="253" reason_ip="109.133.192.1"/>
        <service name="smtp" method="table" conf="3"/></port>"""

        self.port_string8 = """
        <port protocol="tcp" portid="22">
        <service name="smtp" method="table" conf="3"/>
        </port>"""
        self.port_string9 = """
        <port protocol="tcp" portid="22">
        <state/>
        <service name="smtp" method="table" conf="3"/>
        </port>"""

    def test_class_parser(self):
        for testfile in self.flist:
            fd = open(testfile["file"], "r")
            s = fd.read()
            fd.close()
            NmapParser.parse(s)

    def test_class_ports_parser(self):
        pdict = NmapParser.parse(self.ports_string)
        plist = pdict["ports"]
        self.assertEqual(len(plist), 4)
        self.assertEqual(
            sorted([p.port for p in plist]), sorted([22, 25, 9929, 80])
        )
        self.assertRaises(ValueError, NmapParser.parse, self.ports_string2)

    def test_class_port_parser(self):
        p = NmapParser.parse(self.port_string)
        self.assertEqual(p.port, 25)
        self.assertNotEqual(p.state, "open")
        self.assertEqual(p.state, "filtered")
        self.assertEqual(p.service, "smtp")
        self.assertEqual(p.reason, "admin-prohibited")
        self.assertEqual(p.reason_ttl, "253")
        self.assertEqual(p.reason_ip, "109.133.192.1")

    def test_port_except(self):
        self.assertRaises(ValueError, NmapParser.parse, self.port_string2)
        self.assertRaises(
            NmapParserException, NmapParser.parse, self.port_string3
        )
        self.assertRaises(
            NmapParserException, NmapParser.parse, self.port_string4
        )
        self.assertRaises(
            NmapParserException, NmapParser.parse, self.port_string5
        )
        self.assertRaises(ValueError, NmapParser.parse, self.port_string6)
        self.assertRaises(
            NmapParserException, NmapParser.parse, self.port_string7
        )
        self.assertRaises(
            NmapParserException, NmapParser.parse, self.port_string8
        )
        serv = NmapParser.parse(self.port_string9)
        self.assertEqual(serv.state, None)

    def test_parser_generic(self):
        plist = NmapParser.parse_fromstring(self.ports_string)
        for p in plist:
            print(p)


if __name__ == "__main__":
    test_suite = [
        "test_class_parser",
        "test_class_ports_parser",
        "test_class_port_parser",
        "test_port_except",
        "test_parser_generic",
    ]
    suite = unittest.TestSuite(map(TestNmapParser, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



# -*- coding: utf-8 -*-

from libnmap.diff import NmapDiff
from libnmap.objects.os import NmapOSFingerprint


class NmapHost(object):
    """
    NmapHost is a class representing a host object of NmapReport
    """

    def __init__(
        self,
        starttime="",
        endtime="",
        address=None,
        status=None,
        hostnames=None,
        services=None,
        extras=None,
    ):
        """
        NmapHost constructor
        :param starttime: unix timestamp of when the scan against
        that host started
        :type starttime: string
        :param endtime: unix timestamp of when the scan against
        that host ended
        :type endtime: string
        :param address: dict ie :{'addr': '127.0.0.1', 'addrtype': 'ipv4'}
        :param status: dict ie:{'reason': 'localhost-response',
                                'state': 'up'}
        :return: NmapHost:
        """
        self._starttime = starttime
        self._endtime = endtime
        self._hostnames = hostnames if hostnames is not None else []
        self._status = status if status is not None else {}
        self._services = services if services is not None else []
        self._extras = extras if extras is not None else {}
        self._osfingerprinted = False
        self.os = None
        if "os" in self._extras:
            self.os = NmapOSFingerprint(self._extras["os"])
            self._osfingerprinted = True
        else:
            self.os = NmapOSFingerprint({})

        self._ipv4_addr = None
        self._ipv6_addr = None
        self._mac_addr = None
        self._vendor = None
        for addr in address:
            if addr["addrtype"] == "ipv4":
                self._ipv4_addr = addr["addr"]
            elif addr["addrtype"] == "ipv6":
                self._ipv6_addr = addr["addr"]
            elif addr["addrtype"] == "mac":
                self._mac_addr = addr["addr"]
            if "vendor" in addr:
                self._vendor = addr["vendor"]

        self._main_address = self._ipv4_addr or self._ipv6_addr or ""
        self._address = address

    def __eq__(self, other):
        """
        Compare eq NmapHost based on :

            - hostnames
            - address
            - if an associated services has changed

        :return: boolean
        """
        rval = False
        if self.__class__ == other.__class__ and self.id == other.id:
            rval = self.changed(other) == 0
        return rval

    def __ne__(self, other):
        """
        Compare ne NmapHost based on:

            - hostnames
            - address
            - if an associated services has changed

        :return: boolean
        """
        rval = True
        if self.__class__ == other.__class__ and self.id == other.id:
            rval = self.changed(other) > 0
        return rval

    def __repr__(self):
        """
        String representing the object
        :return: string
        """
        return "{0}: [{1} ({2}) - {3}]".format(
            self.__class__.__name__,
            self.address,
            " ".join(self._hostnames),
            self.status,
        )

    def __hash__(self):
        """
        Hash is needed to be able to use our object in sets
        :return: hash
        """
        return (
            hash(self.status)
            ^ hash(self.address)
            ^ hash(self._mac_addr)
            ^ hash(frozenset(self._services))
            ^ hash(frozenset(" ".join(self._hostnames)))
        )

    def changed(self, other):
        """
        return the number of attribute who have changed
        :param other: NmapHost object to compare
        :return int
        """
        return len(self.diff(other).changed())

    @property
    def starttime(self):
        """
        Accessor for the unix timestamp of when the scan was started

        :return: string
        """
        return self._starttime

    @property
    def endtime(self):
        """
        Accessor for the unix timestamp of when the scan ended

        :return: string
        """
        return self._endtime

    @property
    def address(self):
        """
        Accessor for the IP address of the scanned host

        :return: IP address as a string
        """
        return self._main_address

    @address.setter
    def address(self, addrdict):
        """
        Setter for the address dictionnary.

        :param addrdict: valid dict is {'addr': '1.1.1.1',
                                        'addrtype': 'ipv4'}
        """
        if addrdict["addrtype"] == "ipv4":
            self._ipv4_addr = addrdict["addr"]
        elif addrdict["addrtype"] == "ipv6":
            self._ipv6_addr = addrdict["addr"]
        elif addrdict["addrtype"] == "mac":
            self._mac_addr = addrdict["addr"]
        if "vendor" in addrdict:
            self._vendor = addrdict["vendor"]

        self._main_address = self._ipv4_addr or self._ipv6_addr or ""
        self._address = addrdict

    @property
    def ipv4(self):
        """
        Accessor for the IPv4 address of the scanned host

        :return: IPv4 address as a string
        """
        return self._ipv4_addr or ""

    @property
    def mac(self):
        """
        Accessor for the MAC address of the scanned host

        :return: MAC address as a string
        """
        return self._mac_addr or ""

    @property
    def vendor(self):
        """
        Accessor for the vendor attribute of the scanned host

        :return: string (vendor) of empty string if no vendor defined
        """
        return self._vendor or ""

    @property
    def ipv6(self):
        """
        Accessor for the IPv6 address of the scanned host

        :return: IPv6 address as a string
        """
        return self._ipv6_addr or ""

    @property
    def status(self):
        """
        Accessor for the host's status (up, down, unknown...)

        :return: string
        """
        return self._status["state"]

    @status.setter
    def status(self, statusdict):
        """
        Setter for the status dictionnary.

        :param statusdict: valid dict is {"state": "open",
                                          "reason": "syn-ack",
                                          "reason_ttl": "0"}
                            'state' is the only mandatory key.
        """
        self._status = statusdict

    def is_up(self):
        """
        method to determine if host is up or not

        :return: bool
        """
        rval = False
        if self.status == "up":
            rval = True
        return rval

    @property
    def hostnames(self):
        """
        Accessor returning the list of hostnames (array of strings).

        :return: array of string
        """
        return self._hostnames

    @property
    def services(self):
        """
        Accessor for the array of scanned services for that host.

        An array of NmapService objects is returned.

        :return: array of NmapService
        """
        return self._services

    def get_ports(self):
        """
        Retrieve a list of the port used by each service of the NmapHost

        :return: list: of tuples (port,'proto') ie:[(22,'tcp'),(25, 'tcp')]
        """
        return [(p.port, p.protocol) for p in self._services]

    def get_open_ports(self):
        """
        Same as get_ports() but only for open ports

        :return: list: of tuples (port,'proto') ie:[(22,'tcp'),(25, 'tcp')]
        """
        return [
            (p.port, p.protocol) for p in self._services if p.state == "open"
        ]

    def get_service(self, portno, protocol="tcp"):
        """
        :param portno: int the portnumber
        :param protocol='tcp': string ('tcp','udp')

        :return: NmapService or None
        """
        plist = [
            p
            for p in self._services
            if (p.port == portno and p.protocol == protocol)
        ]
        if len(plist) > 1:
            raise Exception("Duplicate services found in NmapHost object")
        return plist.pop() if len(plist) else None

    def get_service_byid(self, service_id):
        """
        Returns a NmapService by providing its id.

        The id of a nmap service is a python tupl made of (protocol, port)
        """
        rval = None
        for _tmpservice in self._services:
            if _tmpservice.id == service_id:
                rval = _tmpservice
        return rval

    def os_class_probabilities(self):
        """
        Returns an array of possible OS class detected during
        the OS fingerprinting.

        :return: Array of NmapOSClass objects
        """
        rval = []
        if self.os is not None:
            rval = self.os.osclasses
        return rval

    def os_match_probabilities(self):
        """
        Returns an array of possible OS match detected during
        the OS fingerprinting

        :return: array of NmapOSMatches objects
        """
        rval = []
        if self.os is not None:
            rval = self.os.osmatches
        return rval

    @property
    def os_fingerprinted(self):
        """
        Specify if the host has OS fingerprint data available

        :return: Boolean
        """
        return self._osfingerprinted

    @property
    def os_fingerprint(self):
        """
        Returns the fingerprint of the scanned system.

        :return: string
        """
        rval = ""
        if self.os is not None:
            rval = "\n".join(self.os.fingerprints)
        return rval

    def os_ports_used(self):
        """
        Returns an array of the ports used for OS fingerprinting

        :return: array of ports used: [{'portid': '22',
                                        'proto': 'tcp',
                                        'state': 'open'},]
        """
        rval = []
        try:
            rval = self._extras["os"]["ports_used"]
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def tcpsequence(self):
        """
        Returns the difficulty to determine remotely predict
        the tcp sequencing.

        return: string
        """
        rval = ""
        try:
            rval = self._extras["tcpsequence"]["difficulty"]
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def ipsequence(self):
        """
        Return the class of ip sequence of the remote hosts.

        :return: string
        """
        rval = ""
        try:
            rval = self._extras["ipidsequence"]["class"]
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def uptime(self):
        """
        uptime of the remote host (if nmap was able to determine it)

        :return: string (in seconds)
        """
        rval = 0
        try:
            rval = int(self._extras["uptime"]["seconds"])
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def lastboot(self):
        """
        Since when the host was booted.

        :return: string
        """
        rval = ""
        try:
            rval = self._extras["uptime"]["lastboot"]
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def distance(self):
        """
        Number of hops to host

        :return: int
        """
        rval = 0
        try:
            rval = int(self._extras["distance"]["value"])
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def scripts_results(self):
        """
        Scripts results specific to the scanned host

        :return: array of <script> dictionary
        """
        rval = {}
        try:
            rval = self._extras["hostscript"]
        except (KeyError, TypeError):
            pass
        return rval

    @property
    def id(self):
        """
        id of the host. Used for diff()ing NmapObjects

        :return: string
        """
        return self.address

    @property
    def extraports_state(self):
        """
        dictionnary containing state and amount of extra ports scanned
        for which a common state, usually, closed was discovered.

        :return: dict with keys 'state' and 'count' or None
        """
        _xtrports = self._extras.get("extraports", None)

        if _xtrports is None:
            return None

        return {"state": _xtrports["state"], "count": _xtrports["count"]}

    @property
    def extraports_reasons(self):
        """
        dictionnary containing reasons why extra ports scanned
        for which a common state, usually, closed was discovered.

        :return: array of dict containing keys 'state' and 'count' or None
        """
        r = self._extras.get("extraports", {})

        if r is None:
            return None

        return r.get("reasons", None)

    def get_dict(self):
        """
        Return a dict representation of the object.

        This is needed by NmapDiff to allow comparaison

        :return dict
        """
        d = dict(
            [
                ("{0}::{1}".format(s.__class__.__name__, str(s.id)), hash(s))
                for s in self.services
            ]
        )

        d.update(
            {
                "address": self.address,
                "status": self.status,
                "hostnames": " ".join(self._hostnames),
                "mac_addr": self._mac_addr,
            }
        )

        return d

    def diff(self, other):
        """
        Calls NmapDiff to check the difference between self and
        another NmapHost object.

        Will return a NmapDiff object.

        This objects return python set() of keys describing the elements
        which have changed, were added, removed or kept unchanged.

        :param other: NmapHost to diff with

        :return: NmapDiff object
        """
        return NmapDiff(self, other)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.parser import NmapParser


class TestNmapFP(unittest.TestCase):
    def setUp(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        self.flist_full = [
            {
                "file": "%s/%s" % (fdir, "files/1_os_banner_scripts.xml"),
                "os": 1,
            },
            {"file": "%s/%s" % (fdir, "files/2_hosts_version.xml"), "os": 1},
            {
                "file": "%s/%s"
                % (fdir, "files/1_hosts_banner_ports_notsyn.xml"),
                "os": 0,
            },
            {"file": "%s/%s" % (fdir, "files/1_hosts_banner.xml"), "os": 0},
            {"file": "%s/%s" % (fdir, "files/1_hosts_down.xml"), "os": 0},
        ]
        self.flist = self.flist_full
        self.flist_os = {
            "nv6": {"file": "%s/%s" % (fdir, "files/full_sudo6.xml"), "os": 0},
            "fullscan": {
                "file": "%s/%s" % (fdir, "files/fullscan.xml"),
                "os": 0,
            },
            "nv5": {"file": "%s/%s" % (fdir, "files/os_scan5.xml"), "os": 0},
        }
        self.fos_class_probabilities = "{0}/{1}".format(
            fdir, "files/test_osclass.xml"
        )

    def test_fp(self):
        for file_e in self.flist_full:
            rep = NmapParser.parse_fromfile(file_e["file"])
            for _host in rep.hosts:
                if file_e["os"] != 0:
                    self.assertTrue(_host.os_fingerprinted)
                elif file_e["os"] == 0:
                    self.assertFalse(_host.os_fingerprinted)
                else:
                    raise Exception

    def test_osclasses_new(self):
        oclines = [
            [
                [
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Apple",
                        "osfamily": "Mac OS X",
                        "osgen": "10.8.X",
                    },
                    {
                        "type": "phone",
                        "accuracy": 100,
                        "vendor": "Apple",
                        "osfamily": "iOS",
                        "osgen": "5.X",
                    },
                    {
                        "type": "media device",
                        "accuracy": 100,
                        "vendor": "Apple",
                        "osfamily": "iOS",
                        "osgen": "5.X",
                    },
                ]
            ],
            [
                [
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "2008",
                    }
                ],
                [
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "7",
                    }
                ],
                [
                    {
                        "type": "phone",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "Phone",
                    }
                ],
                [
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "Vista",
                    },
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "2008",
                    },
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "7",
                    },
                ],
                [
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "Vista",
                    },
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "7",
                    },
                    {
                        "type": "general purpose",
                        "accuracy": 100,
                        "vendor": "Microsoft",
                        "osfamily": "Windows",
                        "osgen": "2008",
                    },
                ],
            ],
        ]
        rep = NmapParser.parse_fromfile(self.flist_os["nv6"]["file"])
        hlist = []
        hlist.append(rep.hosts.pop())
        hlist.append(rep.hosts.pop())
        i = 0
        j = 0
        k = 0
        for h in hlist:
            for om in h.os.osmatches:
                for oc in om.osclasses:
                    tdict = {
                        "type": oc.type,
                        "accuracy": oc.accuracy,
                        "vendor": oc.vendor,
                        "osfamily": oc.osfamily,
                        "osgen": oc.osgen,
                    }
                    self.assertEqual(oclines[i][j][k], tdict)
                    k += 1
                j += 1
                k = 0
            j = 0
            i += 1

    def test_osmatches_new(self):
        rep = NmapParser.parse_fromfile(self.flist_os["nv6"]["file"])
        hlist = []
        hlist.append(rep.hosts.pop())
        hlist.append(rep.hosts.pop())

        baseline = [
            [
                {
                    "line": 6014,
                    "accuracy": 100,
                    "name": "Apple Mac OS X 10.8 - 10.8.1 (Mountain Lion) (Darwin 12.0.0 - 12.1.0) or iOS 5.0.1",
                }
            ],
            [
                {
                    "line": 52037,
                    "accuracy": 100,
                    "name": "Microsoft Windows Server 2008 Beta 3",
                },
                {
                    "line": 52938,
                    "accuracy": 100,
                    "name": "Microsoft Windows 7 Professional",
                },
                {
                    "line": 54362,
                    "accuracy": 100,
                    "name": "Microsoft Windows Phone 7.5",
                },
                {
                    "line": 54897,
                    "accuracy": 100,
                    "name": "Microsoft Windows Vista SP0 or SP1, Windows Server 2008 SP1, or Windows 7",
                },
                {
                    "line": 55210,
                    "accuracy": 100,
                    "name": "Microsoft Windows Vista SP2, Windows 7 SP1, or Windows Server 2008",
                },
            ],
        ]
        i = 0
        j = 0
        for h in hlist:
            for om in h.os.osmatches:
                tdict = {
                    "line": om.line,
                    "accuracy": om.accuracy,
                    "name": om.name,
                }
                self.assertEqual(baseline[i][j], tdict)
                j += 1
            j = 0
            i += 1

    def test_osmatches_old(self):
        rep = NmapParser.parse_fromfile(self.flist_os["nv5"]["file"])
        h1 = rep.hosts[4]
        h1osmatches = [
            {
                "line": -1,
                "accuracy": 95,
                "name": "general purpose:Linux:Linux",
            },
            {"line": -1, "accuracy": 90, "name": "WAP:Gemtek:embedded"},
            {
                "line": -1,
                "accuracy": 89,
                "name": "general purpose:Nokia:Linux",
            },
            {"line": -1, "accuracy": 88, "name": "webcam:AXIS:Linux"},
        ]

        j = 0
        for om in h1.os.osmatches:
            tdict = {"line": om.line, "accuracy": om.accuracy, "name": om.name}
            self.assertEqual(h1osmatches[j], tdict)
            j += 1

    def test_fpv6(self):
        fpval = "OS:SCAN(V=6.40-2%E=4%D=5/9%OT=88%CT=%CU=%PV=Y%DS=0%DC=L%G=N%TM=536BFF2F%P=x\nOS:86_64-apple-darwin10.8.0)SEQ(SP=F9%GCD=1%ISR=103%TI=RD%TS=A)OPS(O1=M3FD8\nOS:NW4NNT11SLL%O2=M3FD8NW4NNT11SLL%O3=M3FD8NW4NNT11%O4=M3FD8NW4NNT11SLL%O5=\nOS:M3FD8NW4NNT11SLL%O6=M3FD8NNT11SLL)WIN(W1=FFFF%W2=FFFF%W3=FFFF%W4=FFFF%W5\nOS:=FFFF%W6=FFFF)ECN(R=Y%DF=Y%TG=40%W=FFFF%O=M3FD8NW4SLL%CC=N%Q=)T1(R=Y%DF=\nOS:Y%TG=40%S=O%A=S+%F=AS%RD=0%Q=)T2(R=N)T3(R=N)T4(R=Y%DF=Y%TG=40%W=0%S=A%A=\nOS:Z%F=R%O=%RD=0%Q=)U1(R=N)IE(R=N)\n"
        fparray = [
            "OS:SCAN(V=6.40-2%E=4%D=5/9%OT=88%CT=%CU=%PV=Y%DS=0%DC=L%G=N%TM=536BFF2F%P=x\nOS:86_64-apple-darwin10.8.0)SEQ(SP=F9%GCD=1%ISR=103%TI=RD%TS=A)OPS(O1=M3FD8\nOS:NW4NNT11SLL%O2=M3FD8NW4NNT11SLL%O3=M3FD8NW4NNT11%O4=M3FD8NW4NNT11SLL%O5=\nOS:M3FD8NW4NNT11SLL%O6=M3FD8NNT11SLL)WIN(W1=FFFF%W2=FFFF%W3=FFFF%W4=FFFF%W5\nOS:=FFFF%W6=FFFF)ECN(R=Y%DF=Y%TG=40%W=FFFF%O=M3FD8NW4SLL%CC=N%Q=)T1(R=Y%DF=\nOS:Y%TG=40%S=O%A=S+%F=AS%RD=0%Q=)T2(R=N)T3(R=N)T4(R=Y%DF=Y%TG=40%W=0%S=A%A=\nOS:Z%F=R%O=%RD=0%Q=)U1(R=N)IE(R=N)\n"
        ]
        rep = NmapParser.parse_fromfile(self.flist_os["nv6"]["file"])
        h1 = rep.hosts.pop()
        self.assertEqual(h1.os.fingerprint, fpval)
        self.assertEqual(h1.os.fingerprints, fparray)

    def test_fpv5(self):
        fpval = "OS:SCAN(V=5.21%D=5/8%OT=22%CT=1%CU=37884%PV=Y%DS=0%DC=L%G=Y%TM=536BFE32%P=x\nOS:86_64-unknown-linux-gnu)SEQ(SP=100%GCD=1%ISR=106%TI=Z%CI=Z%II=I%TS=8)SEQ\nOS:(SP=101%GCD=1%ISR=107%TI=Z%CI=Z%II=I%TS=8)OPS(O1=M400CST11NW3%O2=M400CST\nOS:11NW3%O3=M400CNNT11NW3%O4=M400CST11NW3%O5=M400CST11NW3%O6=M400CST11)WIN(\nOS:W1=8000%W2=8000%W3=8000%W4=8000%W5=8000%W6=8000)ECN(R=Y%DF=Y%T=40%W=8018\nOS:%O=M400CNNSNW3%CC=Y%Q=)T1(R=Y%DF=Y%T=40%S=O%A=S+%F=AS%RD=0%Q=)T2(R=N)T3(\nOS:R=Y%DF=Y%T=40%W=8000%S=O%A=S+%F=AS%O=M400CST11NW3%RD=0%Q=)T4(R=Y%DF=Y%T=\nOS:40%W=0%S=A%A=Z%F=R%O=%RD=0%Q=)T5(R=Y%DF=Y%T=40%W=0%S=Z%A=S+%F=AR%O=%RD=0\nOS:%Q=)T6(R=Y%DF=Y%T=40%W=0%S=A%A=Z%F=R%O=%RD=0%Q=)T7(R=Y%DF=Y%T=40%W=0%S=Z\nOS:%A=S+%F=AR%O=%RD=0%Q=)U1(R=Y%DF=N%T=40%IPL=164%UN=0%RIPL=G%RID=G%RIPCK=G\nOS:%RUCK=G%RUD=G)IE(R=Y%DFI=N%T=40%CD=S)\n"
        fparray = [
            "OS:SCAN(V=5.21%D=5/8%OT=22%CT=1%CU=37884%PV=Y%DS=0%DC=L%G=Y%TM=536BFE32%P=x\nOS:86_64-unknown-linux-gnu)SEQ(SP=100%GCD=1%ISR=106%TI=Z%CI=Z%II=I%TS=8)SEQ\nOS:(SP=101%GCD=1%ISR=107%TI=Z%CI=Z%II=I%TS=8)OPS(O1=M400CST11NW3%O2=M400CST\nOS:11NW3%O3=M400CNNT11NW3%O4=M400CST11NW3%O5=M400CST11NW3%O6=M400CST11)WIN(\nOS:W1=8000%W2=8000%W3=8000%W4=8000%W5=8000%W6=8000)ECN(R=Y%DF=Y%T=40%W=8018\nOS:%O=M400CNNSNW3%CC=Y%Q=)T1(R=Y%DF=Y%T=40%S=O%A=S+%F=AS%RD=0%Q=)T2(R=N)T3(\nOS:R=Y%DF=Y%T=40%W=8000%S=O%A=S+%F=AS%O=M400CST11NW3%RD=0%Q=)T4(R=Y%DF=Y%T=\nOS:40%W=0%S=A%A=Z%F=R%O=%RD=0%Q=)T5(R=Y%DF=Y%T=40%W=0%S=Z%A=S+%F=AR%O=%RD=0\nOS:%Q=)T6(R=Y%DF=Y%T=40%W=0%S=A%A=Z%F=R%O=%RD=0%Q=)T7(R=Y%DF=Y%T=40%W=0%S=Z\nOS:%A=S+%F=AR%O=%RD=0%Q=)U1(R=Y%DF=N%T=40%IPL=164%UN=0%RIPL=G%RID=G%RIPCK=G\nOS:%RUCK=G%RUD=G)IE(R=Y%DFI=N%T=40%CD=S)\n"
        ]
        rep = NmapParser.parse_fromfile(self.flist_os["nv5"]["file"])
        h1 = rep.hosts[4]
        self.assertEqual(h1.os.fingerprint, fpval)
        self.assertEqual(h1.os.fingerprints, fparray)

    def test_cpeservice(self):
        cpelist = ["cpe:/a:openbsd:openssh:5.9p1", "cpe:/o:linux:linux_kernel"]
        rep = NmapParser.parse_fromfile(self.flist_os["fullscan"]["file"])
        h1 = rep.hosts.pop()
        s = h1.services[0]
        self.assertEqual(s.cpelist[0].cpestring, cpelist[0])
        self.assertEqual(s.cpelist[1].cpestring, cpelist[1])

    def test_os_class_probabilities(self):
        p = NmapParser.parse_fromfile(self.fos_class_probabilities)
        h = p.hosts.pop()
        osc = h.os_class_probabilities().pop()
        self.assertEqual(osc.type, "general purpose")
        self.assertEqual(osc.vendor, "Linux")
        self.assertEqual(osc.osfamily, "Linux")
        self.assertEqual(osc.osgen, "3.X")
        self.assertEqual(osc.accuracy, 100)

        # <osclass type="general purpose" vendor="Linux" osfamily="Linux" osgen="3.X" accuracy="100"><cpe>cpe:/o:linux:linux_kernel:3</cpe></osclass>


if __name__ == "__main__":
    test_suite = [
        "test_fp",
        "test_fpv6",
        "test_osmatches_new",
        "test_osclasses_new",
        "test_fpv5",
        "test_osmatches_old",
        "test_cpeservice",
        "test_os_class_probabilities",
    ]
    suite = unittest.TestSuite(map(TestNmapFP, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python

from time import sleep

from libnmap.process import NmapProcess


def make_nmproc_obj(targets, options):
    return NmapProcess(targets=targets, options=options)


def start_all_bg(nmprocs):
    for nmp in nmprocs:
        nmp.run_background()


def any_running(nmprocs):
    return any([nmp.is_running() for nmp in nmprocs])


def summarize(nmprocs):
    for nmp in nmprocs:
        print(
            "rc: {0} output: {1} stdout len: {2}".format(
                nmp.rc, nmp.summary, len(nmp.stdout)
            )
        )


nm_targets = []
for h in range(10):
    nm_targets.append("scanme.nmap.org")
nm_opts = "-sT"

nm_procs = [make_nmproc_obj(t, nm_opts) for t in nm_targets]
start_all_bg(nm_procs)

while any_running(nm_procs):
    print("Nmap Scan running...")
    sleep(2)

summarize(nm_procs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.parser import NmapParser


class TestNmapReportDiff(unittest.TestCase):
    def setUp(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        self.flist_full = [
            {"file": "%s/%s" % (fdir, "files/2_hosts.xml"), "hosts": 2},
            {"file": "%s/%s" % (fdir, "files/1_hosts.xml"), "hosts": 1},
        ]
        self.flist = self.flist_full

    def test_diff_host_list(self):
        fdir = os.path.dirname(os.path.realpath(__file__))
        r1 = NmapParser.parse_fromfile("%s/%s" % (fdir, "files/1_hosts.xml"))
        r2 = NmapParser.parse_fromfile("%s/%s" % (fdir, "files/2_hosts.xml"))
        r3 = NmapParser.parse_fromfile("%s/%s" % (fdir, "files/1_hosts.xml"))
        r4 = NmapParser.parse_fromfile(
            "%s/%s" % (fdir, "files/2_hosts_achange.xml")
        )

        d1 = r1.diff(r2)
        self.assertEqual(
            d1.changed(),
            set(
                [
                    "hosts_total",
                    "commandline",
                    "hosts_up",
                    "scan_type",
                    "elapsed",
                ]
            ),
        )
        self.assertEqual(
            d1.unchanged(),
            set(["hosts_down", "version", "NmapHost::127.0.0.1"]),
        )
        self.assertEqual(d1.removed(), set(["NmapHost::74.207.244.221"]))

        d2 = r1.diff(r3)
        self.assertEqual(d2.changed(), set([]))
        self.assertEqual(
            d2.unchanged(),
            set(
                [
                    "hosts_total",
                    "commandline",
                    "hosts_up",
                    "NmapHost::127.0.0.1",
                    "elapsed",
                    "version",
                    "scan_type",
                    "hosts_down",
                ]
            ),
        )
        self.assertEqual(d2.added(), set([]))
        self.assertEqual(d2.removed(), set([]))

        d3 = r2.diff(r4)
        self.assertEqual(d3.changed(), set(["NmapHost::127.0.0.1"]))
        self.assertEqual(
            d3.unchanged(),
            set(
                [
                    "hosts_total",
                    "commandline",
                    "hosts_up",
                    "NmapHost::74.207.244.221",
                    "version",
                    "elapsed",
                    "scan_type",
                    "hosts_down",
                ]
            ),
        )
        self.assertEqual(d3.added(), set([]))
        self.assertEqual(d3.removed(), set([]))


if __name__ == "__main__":
    test_suite = ["test_diff_host_list"]
    suite = unittest.TestSuite(map(TestNmapReportDiff, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from libnmap.process import NmapProcess


def mycallback(nmaptask):
    nmaptask = nmap_proc.current_task
    if nmaptask:
        print(
            "Task {0} ({1}): ETC: {2} DONE: {3}%".format(
                nmaptask.name, nmaptask.status, nmaptask.etc, nmaptask.progress
            )
        )


nmap_proc = NmapProcess(
    targets="scanme.nmap.org", options="-sV", event_callback=mycallback
)
nmap_proc.run()
print(nmap_proc.stdout)
print(nmap_proc.stderr)



# -*- coding: utf-8 -*-
from distutils.core import setup

with open("README.rst", encoding="utf-8") as rfile:
    long_description = rfile.read()

setup(
    name="python-libnmap",
    version="0.7.3",
    author="Ronald Bister",
    author_email="mini.pelle@gmail.com",
    packages=["libnmap", "libnmap.plugins", "libnmap.objects"],
    url="http://pypi.python.org/pypi/python-libnmap/",
    extras_require={
        "defusedxml": ["defusedxml>=0.6.0"],
    },
    license="Apache 2.0",
    description=(
        "Python NMAP library enabling you to start async nmap tasks, "
        "parse and compare/diff scan results"
    ),
    long_description=long_description,
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: System :: Networking",
    ],
)



#!/usr/bin/env python

from libnmap.process import NmapProcess


def make_nmproc_obj(targets, options):
    return NmapProcess(targets=targets, options=options)


def start_all(nmprocs):
    for nmp in nmprocs:
        print("Starting scan for host {0}".format(nmp.targets))
        nmp.run()


def summarize(nmprocs):
    for nmp in nmprocs:
        print("rc: {0} output: {1}".format(nmp.rc, len(nmp.stdout)))


nm_targets = []
for h in range(20):
    nm_targets.append("localhost")
nm_opts = "-sT"

nm_procs = [make_nmproc_obj(t, nm_opts) for t in nm_targets]
start_all(nm_procs)

summarize(nm_procs)



# -*- coding: utf-8 -*-

import warnings

from libnmap.objects.cpe import CPE


class OSFPPortUsed(object):
    """
    Port used class: this enables the user of NmapOSFingerprint class
    to have a common and clear interface to access portused data which
    were collected and used during os fingerprint scan
    """

    def __init__(self, port_used_dict):
        try:
            self._state = port_used_dict["state"]
            self._proto = port_used_dict["proto"]
            self._portid = port_used_dict["portid"]
        except KeyError:
            raise Exception("Cannot create OSFPPortUsed: missing required key")

    @property
    def state(self):
        """
        Accessor for the portused state (closed, open,...)
        """
        return self._state

    @property
    def proto(self):
        """
        Accessor for the portused protocol (tcp, udp,...)
        """
        return self._proto

    @property
    def portid(self):
        """
        Accessor for the referenced port number used
        """
        return self._portid


class NmapOSMatch(object):
    """
    NmapOSMatch is an internal class used for offering results
    from an nmap os fingerprint. This common interfaces makes
    a compatibility between old nmap xml (<1.04) and new nmap
    xml versions (used in nmapv6 for instance).

    In previous xml version, osclass tags from nmap fingerprints
    were not directly mapped to a osmatch. In new xml version,
    osclass could be embedded in osmatch tag.

    The approach to solve this is to create a common class
    which will, for older xml version, match based on the accuracy
    osclass to an osmatch. If no match, an osmatch will be made up
    from a concat of os class attributes: vendor and osfamily.
    Unmatched osclass will have a line attribute of -1.

    More info, see issue #26 or http://seclists.org/nmap-dev/2012/q2/252
    """

    def __init__(self, osmatch_dict):
        _osmatch_dict = osmatch_dict["osmatch"]
        if (
            "name" not in _osmatch_dict
            or "line" not in _osmatch_dict
            or "accuracy" not in _osmatch_dict
        ):
            raise Exception("Cannot create NmapOSClass: missing required key")

        self._name = _osmatch_dict["name"]
        self._line = _osmatch_dict["line"]
        self._accuracy = _osmatch_dict["accuracy"]

        # create osclass list
        self._osclasses = []
        try:
            for _osclass in osmatch_dict["osclasses"]:
                try:
                    _osclassobj = NmapOSClass(_osclass)
                except Exception as e:
                    em = "Could not create NmapOSClass object: {0}".format(e)
                    raise Exception(em)
                self._osclasses.append(_osclassobj)
        except KeyError:
            pass

    def add_osclass(self, osclass_obj):

        """
        Add a NmapOSClass object to the OSMatch object. This method is
        useful to implement compatibility with older versions of NMAP
        by providing a common interface to access os fingerprint data.
        """
        self._osclasses.append(osclass_obj)

    @property
    def osclasses(self):
        """
        Accessor for all NmapOSClass objects matching with this OS Match
        """
        return self._osclasses

    @property
    def name(self):
        """
        Accessor for name attribute (e.g.: Linux 2.4.26 (Slackware 10.0.0))
        """
        return self._name

    @property
    def line(self):
        """
        Accessor for line attribute as integer. value equals -1 if this
        osmatch holds orphans NmapOSClass objects. This could happen with
        older version of nmap xml engine (<1.04 (e.g: nmapv6)).

        :return: int
        """
        return int(self._line)

    @property
    def accuracy(self):
        """
        Accessor for accuracy

        :return: int
        """
        return int(self._accuracy)

    def get_cpe(self):
        """
        This method return a list of cpe stings and not CPE objects as
        the NmapOSClass.cpelist property. This method is a helper to
        simplify data management.

        For more advanced handling of CPE data, use NmapOSClass.cpelist
        and use the methods from CPE class
        """
        _cpelist = []
        for osc in self.osclasses:
            for cpe in osc.cpelist:
                _cpelist.append(cpe.cpestring)
        return _cpelist

    def __repr__(self):
        rval = "{0}: {1}".format(self.name, self.accuracy)
        for _osclass in self._osclasses:
            rval += "\r\n  |__ os class: {0}".format(str(_osclass))
        return rval


class NmapOSClass(object):
    """
    NmapOSClass offers an unified API to access data from analysed
    osclass tag. As implemented in libnmap and newer version of nmap,
    osclass objects will always be embedded in a NmapOSMatch.
    Unmatched NmapOSClass will be stored in "dummy" NmapOSMatch objects
    which will have the particularity of have a line attribute of -1.
    On top of this, NmapOSClass will have optional CPE objects
    embedded.
    """

    def __init__(self, osclass_dict):
        _osclass = osclass_dict["osclass"]
        if (
            "vendor" not in _osclass
            or "osfamily" not in _osclass
            or "accuracy" not in _osclass
        ):
            raise Exception("Wrong osclass structure: missing required key")

        self._vendor = _osclass["vendor"]
        self._osfamily = _osclass["osfamily"]
        self._accuracy = _osclass["accuracy"]

        self._osgen = ""
        self._type = ""
        # optional data
        if "osgen" in _osclass:
            self._osgen = _osclass["osgen"]
        if "type" in _osclass:
            self._type = _osclass["type"]

        self._cpelist = []
        for _cpe in osclass_dict["cpe"]:
            self._cpelist.append(CPE(_cpe))

    @property
    def cpelist(self):
        """
        Returns a list of CPE Objects matching with this os class

        :return: list of CPE objects
        :rtype: Array
        """
        return self._cpelist

    @property
    def vendor(self):
        """
        Accessor for vendor information (Microsoft, Linux,...)

        :return: string
        """
        return self._vendor

    @property
    def osfamily(self):
        """
        Accessor for OS family information (Windows, Linux,...)

        :return: string
        """
        return self._osfamily

    @property
    def accuracy(self):
        """
        Accessor for OS class detection accuracy (int)

        :return: int
        """
        return int(self._accuracy)

    @property
    def osgen(self):
        """
        Accessor for OS class generation (7, 8, 2.4.X,...).

        :return: string
        """
        return self._osgen

    @property
    def type(self):
        """
        Accessor for OS class type (general purpose,...)

        :return: string
        """
        return self._type

    @property
    def description(self):
        """
        Accessor helper which returns a concataned string of
        the valuable attributes from NmapOSClass object

        :return: string
        """
        rval = "{0}: {1}, {2}".format(self.type, self.vendor, self.osfamily)
        if len(self.osgen):
            rval += "({0})".format(self.osgen)
        return rval

    def __repr__(self):
        rval = "{0}: {1}, {2}".format(self.type, self.vendor, self.osfamily)
        if len(self.osgen):
            rval += "({0})".format(self.osgen)
        for _cpe in self._cpelist:
            rval += "\r\n    |__ {0}".format(str(_cpe))
        return rval


class NmapOSFingerprint(object):
    """
    NmapOSFingerprint is a easier API for using os fingerprinting.
    Data for OS fingerprint (<os> tag) is instantiated from
    a NmapOSFingerprint which is accessible in NmapHost via NmapHost.os
    """

    def __init__(self, osfp_data):
        self.__osmatches = []
        self.__ports_used = []
        self.__fingerprints = []

        if "osmatches" in osfp_data:
            for _osmatch in osfp_data["osmatches"]:
                _osmatch_obj = NmapOSMatch(_osmatch)
                self.__osmatches.append(_osmatch_obj)
        if "osclasses" in osfp_data:
            for _osclass in osfp_data["osclasses"]:
                _osclass_obj = NmapOSClass(_osclass)
                _osmatched = self.get_osmatch(_osclass_obj)
                if _osmatched is not None:
                    _osmatched.add_osclass(_osclass_obj)
                else:
                    self._add_dummy_osmatch(_osclass_obj)
        if "osfingerprints" in osfp_data:
            for _osfp in osfp_data["osfingerprints"]:
                if "fingerprint" in _osfp:
                    self.__fingerprints.append(_osfp["fingerprint"])
        if "ports_used" in osfp_data:
            for _pused_dict in osfp_data["ports_used"]:
                _pused = OSFPPortUsed(_pused_dict)
                self.__ports_used.append(_pused)

    def get_osmatch(self, osclass_obj):
        """
        This function enables NmapOSFingerprint to determine if an
        NmapOSClass object could be attached to an existing NmapOSMatch
        object in order to respect the common interface for
        the nmap xml version < 1.04 and >= 1.04

        This method will return an NmapOSMatch object matching with
        the NmapOSClass provided in parameter
        (match is performed based on accuracy)

        :return: NmapOSMatch object
        """
        rval = None
        for _osmatch in self.__osmatches:
            if _osmatch.accuracy == osclass_obj.accuracy:
                rval = _osmatch
                break  # sorry
        return rval

    def _add_dummy_osmatch(self, osclass_obj):
        """
        This functions creates a dummy NmapOSMatch object in order to
        encapsulate an NmapOSClass object which was not matched with an
        existing NmapOSMatch object
        """
        _dname = "{0}:{1}:{2}".format(
            osclass_obj.type, osclass_obj.vendor, osclass_obj.osfamily
        )
        _dummy_dict = {
            "osmatch": {
                "name": _dname,
                "accuracy": osclass_obj.accuracy,
                "line": -1,
            },
            "osclasses": [],
        }
        _dummy_osmatch = NmapOSMatch(_dummy_dict)
        self.__osmatches.append(_dummy_osmatch)

    @property
    def osmatches(self, min_accuracy=0):
        _osmatches = []

        for _osmatch in self.__osmatches:
            if _osmatch.accuracy >= min_accuracy:
                _osmatches.append(_osmatch)

        return _osmatches

    @property
    def osclasses(self, min_accuracy=0):
        osc_array = []
        for _osm in self.osmatches:
            for _osc in _osm.osclasses:
                if _osc.accuracy >= min_accuracy:
                    osc_array.append(_osc)
        return osc_array

    @property
    def fingerprint(self):
        return "\r\n".join(self.__fingerprints)

    @property
    def fingerprints(self):
        return self.__fingerprints

    @property
    def ports_used(self):
        """
        Return an array of OSFPPortUsed object with the ports used to
        perform the os fingerprint. This dict might contain another dict
        embedded containing the ports_reason values.
        """
        return self.__ports_used

    def osmatch(self, min_accuracy=90):
        warnings.warn(
            "NmapOSFingerprint.osmatch is deprecated: "
            "use NmapOSFingerprint.osmatches",
            DeprecationWarning,
        )
        os_array = []
        for _osmatch in self.__osmatches:
            if _osmatch.accuracy >= min_accuracy:
                os_array.append(_osmatch.name)
        return os_array

    def osclass(self, min_accuracy=90):
        warnings.warn(
            "NmapOSFingerprint.osclass() is deprecated: "
            "use NmapOSFingerprint.osclasses() if applicable",
            DeprecationWarning,
        )
        os_array = []
        for osmatch_entry in self.osmatches:
            if osmatch_entry.accuracy >= min_accuracy:
                for oclass in osmatch_entry.osclasses:
                    _ftstr = "type:{0}|vendor:{1}|osfamily{2}".format(
                        oclass.type, oclass.vendor, oclass.osfamily
                    )
                    os_array.append(_ftstr)
        return os_array

    def os_cpelist(self):
        cpelist = []
        for _osmatch in self.osmatches:
            for oclass in _osmatch.osclasses:
                cpelist.extend(oclass.cpelist)
        return cpelist

    def __repr__(self):
        rval = ""
        for _osmatch in self.osmatches:
            rval += "\r\n{0}".format(_osmatch)
        rval += "Fingerprints: {0}".format(self.fingerprint)
        return rval



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import unittest

from libnmap.parser import NmapParser

host1 = """
<host starttime="1361738377" endtime="1361738377">
<status state="up" reason="localhost-response"/>
<address addr="127.0.0.1" addrtype="ipv4"/>
<hostnames>
<hostname name="localhost" type="user"/>
<hostname name="localhost" type="PTR"/>
</hostnames>
<ports><extraports state="WILLY_WONCKA" count="995">
<extrareasons reason="conn-refused" count="995"/>
</extraports>
<port protocol="tcp" portid="22">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ssh" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="25">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="smtp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="111">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="rpcbind" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="631">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ipp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="3306">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="mysql" method="table" conf="3"/>
</port>
</ports>
<times srtt="2100" rttvar="688" to="100000"/>
</host>
"""
host2 = """
<host starttime="1361738318" endtime="13617386177">
<status state="up" reason="localhost-respoe"/>
<address addr="127.0.0.1" addrtype="ipv4"/>
<hostnames>
<hostname name="localhost" type="user"/>
<hostname name="localhost" type="PTR"/>
<hostname name="localhost2" type="PTR"/>
</hostnames>
<ports><extraports state="closed" count="995">
<extrareasons reason="conn-refused" count="995"/>
</extraports>
<port protocol="tcp" portid="22">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ssh" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="25">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="smtp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="111">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="rpcbind" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="631">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ipp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="3306">
<state state="open" reason="syn-ack" reason_ttl="1"/>
<service name="mysql" method="table" conf="3"/>
</port>
</ports>
<times srtt="2100" rttvar="688" to="100000"/>
</host>
"""

host3 = """<host starttime="13617" endtime="13617">
<status state="down" reason="localhost-response"/>
<address addr="127.0.0.1" addrtype="ipv4"/>
<hostnames>
<hostname name="localhost" type="user"/>
<hostname name="localhost" type="PTR"/>
</hostnames>
<ports><extraports state="closed" count="995">
<extrareasons reason="conn-refused" count="995"/>
</extraports>
<port protocol="tcp" portid="22">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ssh" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="111">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="rpcbind" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="631">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ipp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="3306">
<state state="closed" reason="syn-ack" reason_ttl="0"/>
<service name="mysql" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="3307">
<state state="closed" reason="syn-ack" reason_ttl="0"/>
<service name="mysql" method="table" conf="3"/>
</port>
</ports>
<times srtt="2100" rttvar="688" to="100000"/>
</host>
"""

host4 = """
<host starttime="77" endtime="13">
<status state="up" reason="locaonse"/>
<address addr="127.0.0.1" addrtype="ipv4"/>
<hostnames>
<hostname name="localhost" type="user"/>
<hostname name="localhost" type="PTR"/>
</hostnames>
<ports><extraports state="azeazeaze" count="995">
<extrareasons reason="conn-refused" count="995"/>
</extraports>
<port protocol="tcp" portid="22">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ssh" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="25">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="smtp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="111">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="rpcbind" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="631">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="ipp" method="table" conf="3"/>
</port>
<port protocol="tcp" portid="3306">
<state state="open" reason="syn-ack" reason_ttl="0"/>
<service name="mysql" method="table" conf="3"/>
</port>
</ports>
<times srtt="200" rttvar="68" to="100"/>
</host>
"""


class TestNmapHost(unittest.TestCase):
    def setUp(self):
        self.fdir = os.path.dirname(os.path.realpath(__file__))
        self.dionaea_path = "{0}/files/dionaea_scan.xml".format(self.fdir)

    def test_eq_host(self):
        h1 = NmapParser.parse(host1)
        h2 = NmapParser.parse(host2)
        h3 = NmapParser.parse(host3)
        h4 = NmapParser.parse(host4)

        self.assertNotEqual(h1, h2)
        self.assertEqual(h1, h1)
        self.assertNotEqual(h1, h3)
        self.assertEqual(h1, h4)
        self.assertNotEqual(h2, h3)

    def test_host_api(self):
        h = NmapParser.parse(host2)
        self.assertEqual(h.starttime, "1361738318")
        self.assertEqual(h.endtime, "13617386177")
        self.assertEqual(h.address, "127.0.0.1")
        self.assertEqual(h.status, "up")
        self.assertEqual(h.hostnames, ["localhost", "localhost", "localhost2"])

        h2 = NmapParser.parse(host3)
        self.assertEqual(len(h2.services), 5)
        self.assertEqual(len(h2.get_ports()), 5)
        self.assertEqual(len(h2.get_open_ports()), 3)
        self.assertEqual(h2.get_service(22, "tcp").state, "open")

    def test_extra_ports(self):
        h1 = NmapParser.parse(host1)
        h2 = NmapParser.parse(host2)

        self.assertEqual(
            h1.extraports_state["state"],
            {"count": "995", "state": "WILLY_WONCKA"},
        )
        self.assertEqual(
            h1.extraports_reasons, [{"reason": "conn-refused", "count": "995"}]
        )

        self.assertEqual(
            h2.extraports_state["state"], {"count": "995", "state": "closed"}
        )
        self.assertEqual(
            h2.extraports_reasons, [{"reason": "conn-refused", "count": "995"}]
        )

    def test_diff_host(self):
        h1 = NmapParser.parse(host1)
        h2 = NmapParser.parse(host2)
        h3 = NmapParser.parse(host3)

        c1 = h1.diff(h2)
        c2 = h1.diff(h3)
        c3 = h2.diff(h3)

        self.assertEqual(c1.changed(), set(["hostnames"]))
        self.assertEqual(c1.added(), set([]))
        self.assertEqual(c1.removed(), set([]))

        self.assertEqual(
            c1.unchanged(),
            set(
                [
                    "status",
                    "NmapService::tcp.22",
                    "NmapService::tcp.111",
                    "NmapService::tcp.631",
                    "NmapService::tcp.3306",
                    "address",
                    "NmapService::tcp.25",
                    "mac_addr",
                ]
            ),
        )

        self.assertEqual(
            c2.changed(), set(["status", "NmapService::tcp.3306"])
        )
        self.assertEqual(c2.added(), set(["NmapService::tcp.25"]))
        self.assertEqual(c2.removed(), set(["NmapService::tcp.3307"]))
        self.assertEqual(
            c2.unchanged(),
            set(
                [
                    "NmapService::tcp.631",
                    "hostnames",
                    "NmapService::tcp.22",
                    "NmapService::tcp.111",
                    "address",
                    "mac_addr",
                ]
            ),
        )

        self.assertEqual(
            c3.changed(), set(["status", "hostnames", "NmapService::tcp.3306"])
        )
        self.assertEqual(c3.added(), set(["NmapService::tcp.25"]))
        self.assertEqual(c3.removed(), set(["NmapService::tcp.3307"]))
        self.assertEqual(
            c3.unchanged(),
            set(
                [
                    "NmapService::tcp.631",
                    "NmapService::tcp.22",
                    "NmapService::tcp.111",
                    "address",
                    "mac_addr",
                ]
            ),
        )


if __name__ == "__main__":
    test_suite = ["test_eq_host", "test_host_api", "test_diff_host"]
    suite = unittest.TestSuite(map(TestNmapHost, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from libnmap.parser import NmapParser, NmapParserException

baddatalist = [
    "<host>aaa",
    None,
    "",
    123,
    "ports/>>>",
    "<port<>",
    "<port/>",
    "<ports/aaaa>",
]


class TestNmapParser(unittest.TestCase):
    def test_parse(self):
        for baddata in baddatalist:
            self.assertRaises(
                NmapParserException, NmapParser.parse, baddata, "zz"
            )
            self.assertRaises(
                NmapParserException, NmapParser.parse, baddata, "XML"
            )
            self.assertRaises(
                NmapParserException, NmapParser.parse, baddata, "YAML"
            )


if __name__ == "__main__":
    test_suite = ["test_parse"]

    suite = unittest.TestSuite(map(TestNmapParser, test_suite))
    test_result = unittest.TextTestRunner(verbosity=2).run(suite)
